/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     ABOUT.cpp
//
// DESCRIPTION
//     About dialogue for SACAMOS
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////
#include "ABOUT.h"

#include <wx/textfile.h>
#include <wx/msgdlg.h>
#include <wx/hyperlink.h>

//(*InternalHeaders(ABOUT)
#include <wx/bitmap.h>
#include <wx/intl.h>
#include <wx/image.h>
#include <wx/string.h>
//*)

//(*IdInit(ABOUT)
const long ABOUT::ID_STATICBITMAP1 = wxNewId();
const long ABOUT::ID_TEXTCTRL2 = wxNewId();
const long ABOUT::ID_TEXTCTRL1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(ABOUT,wxDialog)
	//(*EventTable(ABOUT)
	//*)
END_EVENT_TABLE()

ABOUT::ABOUT(wxWindow* parent,wxWindowID id,const wxPoint& pos,const wxSize& size)
{
	//(*Initialize(ABOUT)
	wxStaticBoxSizer* StaticBoxSizer2;
	wxBoxSizer* BoxSizer1;
	wxStaticBoxSizer* StaticBoxSizer1;
	wxFlexGridSizer* FlexGridSizer1;

	Create(parent, wxID_ANY, _("ABOUT SACAMOS"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("wxID_ANY"));
	FlexGridSizer1 = new wxFlexGridSizer(0, 1, 0, 0);
	BoxSizer1 = new wxBoxSizer(wxVERTICAL);
	StaticBitmap1 = new wxStaticBitmap(this, ID_STATICBITMAP1, wxBitmap(wxImage(_T("resources//Sacamos_About.png"))), wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER, _T("ID_STATICBITMAP1"));
	BoxSizer1->Add(StaticBitmap1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticBoxSizer1 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("BUILD"));
	TextCtrl2 = new wxTextCtrl(this, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_AUTO_URL, wxDefaultValidator, _T("ID_TEXTCTRL2"));
	StaticBoxSizer1->Add(TextCtrl2, 1, wxALL|wxEXPAND, 5);
	BoxSizer1->Add(StaticBoxSizer1, 1, wxALL|wxEXPAND, 5);
	StaticBoxSizer2 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("LICENSE"));
	TextCtrl1 = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_AUTO_URL|wxTE_WORDWRAP, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	StaticBoxSizer2->Add(TextCtrl1, 1, wxALL|wxEXPAND, 5);
	BoxSizer1->Add(StaticBoxSizer2, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer1->Add(BoxSizer1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);

	Connect(ID_TEXTCTRL2,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&ABOUT::OnTextCtrl2Text);
	Connect(ID_TEXTCTRL1,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&ABOUT::OnTextCtrl1Text);
	Connect(wxID_ANY,wxEVT_INIT_DIALOG,(wxObjectEventFunction)&ABOUT::OnInit);
	//*)
}

ABOUT::~ABOUT()
{
	//(*Destroy(ABOUT)
	//*)
}

void ABOUT::DisplayLicenseFile()
{
    wxString LicenseFileName = "resources/License";
    wxString text;
    wxTextFile LicenseFile;

    //int lineCount = LicenseFile.GetLineCount();

    if (LicenseFile.Open(LicenseFileName))
    {
        for (int i = 0; i<int(LicenseFile.GetLineCount()); i++)
        {
            text = text + LicenseFile[i];

            text = text + "\n";
        }
    TextCtrl1->SetValue(text);
    }

}

void ABOUT::DisplayBuildFile()
{
    wxString BuildFileName = "resources/Build";
    wxString text;
    wxTextFile BuildFile;

    if (BuildFile.Open(BuildFileName))
    {
        for (int i = 1; i<4; i++)
        {
            text = text + BuildFile[i];

            text = text + "\n";
        }
    TextCtrl2->SetValue(text);

    }
}

void ABOUT::OnInit(wxInitDialogEvent& event)
{

    DisplayLicenseFile();

    DisplayBuildFile();

}

void ABOUT::OnTextCtrl1Text(wxCommandEvent& event)
{

}

void ABOUT::OnTextCtrl2Text(wxCommandEvent& event)
{
}
