/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     COAXIAL.cpp
//
// DESCRIPTION
//     GUI for the construction of cable_spec file for cable type: Coaxial
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////

#include "COAXIAL.h"

//(*InternalHeaders(COAXIAL)
#include <wx/settings.h>
#include <wx/font.h>
#include <wx/intl.h>
#include <wx/string.h>
//*)

//(*IdInit(COAXIAL)
const long COAXIAL::ID_STATICTEXT1 = wxNewId();
const long COAXIAL::ID_TEXTCTRL1 = wxNewId();
const long COAXIAL::ID_STATICTEXT4 = wxNewId();
const long COAXIAL::ID_TEXTCTRL4 = wxNewId();
const long COAXIAL::ID_STATICTEXT5 = wxNewId();
const long COAXIAL::ID_TEXTCTRL5 = wxNewId();
const long COAXIAL::ID_STATICTEXT6 = wxNewId();
const long COAXIAL::ID_TEXTCTRL6 = wxNewId();
const long COAXIAL::ID_STATICTEXT7 = wxNewId();
const long COAXIAL::ID_TEXTCTRL7 = wxNewId();
const long COAXIAL::ID_STATICTEXT8 = wxNewId();
const long COAXIAL::ID_TEXTCTRL8 = wxNewId();
const long COAXIAL::ID_STATICTEXT9 = wxNewId();
const long COAXIAL::ID_TEXTCTRL9 = wxNewId();
const long COAXIAL::ID_PANEL2 = wxNewId();
const long COAXIAL::ID_STATICTEXT2 = wxNewId();
const long COAXIAL::ID_TEXTCTRL2 = wxNewId();
const long COAXIAL::ID_STATICTEXT3 = wxNewId();
const long COAXIAL::ID_TEXTCTRL3 = wxNewId();
const long COAXIAL::ID_STATICTEXT10 = wxNewId();
const long COAXIAL::ID_STATICTEXT11 = wxNewId();
const long COAXIAL::ID_STATICTEXT12 = wxNewId();
const long COAXIAL::ID_CHECKBOX1 = wxNewId();
const long COAXIAL::ID_PANEL3 = wxNewId();
const long COAXIAL::ID_STATICTEXT18 = wxNewId();
const long COAXIAL::ID_STATICTEXT19 = wxNewId();
const long COAXIAL::ID_STATICTEXT20 = wxNewId();
const long COAXIAL::ID_TEXTCTRL15 = wxNewId();
const long COAXIAL::ID_STATICTEXT21 = wxNewId();
const long COAXIAL::ID_TEXTCTRL16 = wxNewId();
const long COAXIAL::ID_CHECKBOX2 = wxNewId();
const long COAXIAL::ID_STATICTEXT15 = wxNewId();
const long COAXIAL::ID_TEXTCTRL11 = wxNewId();
const long COAXIAL::ID_STATICLINE1 = wxNewId();
const long COAXIAL::ID_STATICLINE2 = wxNewId();
const long COAXIAL::ID_STATICTEXT13 = wxNewId();
const long COAXIAL::ID_TEXTCTRL10 = wxNewId();
const long COAXIAL::ID_STATICTEXT14 = wxNewId();
const long COAXIAL::ID_TEXTCTRL12 = wxNewId();
const long COAXIAL::ID_STATICLINE3 = wxNewId();
const long COAXIAL::ID_STATICLINE4 = wxNewId();
const long COAXIAL::ID_STATICTEXT16 = wxNewId();
const long COAXIAL::ID_TEXTCTRL13 = wxNewId();
const long COAXIAL::ID_STATICTEXT17 = wxNewId();
const long COAXIAL::ID_TEXTCTRL14 = wxNewId();
const long COAXIAL::ID_PANEL4 = wxNewId();
const long COAXIAL::ID_NOTEBOOK1 = wxNewId();
const long COAXIAL::ID_BUTTON1 = wxNewId();
const long COAXIAL::ID_BUTTON2 = wxNewId();
const long COAXIAL::ID_coaxCLOSE = wxNewId();
const long COAXIAL::ID_PANEL1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(COAXIAL,wxDialog)
	//(*EventTable(COAXIAL)
	//*)
END_EVENT_TABLE()

COAXIAL::COAXIAL(wxWindow* parent,wxWindowID id,const wxPoint& pos,const wxSize& size)
{
	//(*Initialize(COAXIAL)
	wxStaticBoxSizer* StaticBoxSizer2;
	wxFlexGridSizer* FlexGridSizer4;
	wxFlexGridSizer* FlexGridSizer3;
	wxFlexGridSizer* FlexGridSizer5;
	wxFlexGridSizer* FlexGridSizer2;
	wxBoxSizer* BoxSizer2;
	wxFlexGridSizer* FlexGridSizer7;
	wxBoxSizer* BoxSizer1;
	wxFlexGridSizer* FlexGridSizer6;
	wxStaticBoxSizer* StaticBoxSizer1;
	wxFlexGridSizer* FlexGridSizer1;
	wxBoxSizer* BoxSizer3;

	Create(parent, wxID_ANY, _("SACAMOS: Coaxial Cable Model"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER|wxCLOSE_BOX|wxMAXIMIZE_BOX|wxMINIMIZE_BOX|wxNO_BORDER, _T("wxID_ANY"));
	FlexGridSizer1 = new wxFlexGridSizer(2, 2, 0, 5);
	FlexGridSizer1->AddGrowableCol(1);
	StaticBoxSizer1 = new wxStaticBoxSizer(wxVERTICAL, this, _("COAXIAL Cable Parameters"));
	FlexGridSizer2 = new wxFlexGridSizer(0, 1, 0, 0);
	BoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
	StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("Cable Name"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
	BoxSizer2->Add(StaticText1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl1 = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	wxFont TextCtrl1Font(10,wxFONTFAMILY_SWISS,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL,false,wxEmptyString,wxFONTENCODING_DEFAULT);
	TextCtrl1->SetFont(TextCtrl1Font);
	TextCtrl1->SetToolTip(_("Cable Filename"));
	BoxSizer2->Add(TextCtrl1, 2, wxALL|wxEXPAND, 5);
	FlexGridSizer2->Add(BoxSizer2, 1, wxALL|wxEXPAND, 5);
	BoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
	Notebook1 = new wxNotebook(this, ID_NOTEBOOK1, wxDefaultPosition, wxDefaultSize, 0, _T("ID_NOTEBOOK1"));
	Panel2 = new wxPanel(Notebook1, ID_PANEL2, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANEL2"));
	FlexGridSizer3 = new wxFlexGridSizer(0, 2, 0, 0);
	FlexGridSizer3->AddGrowableCol(0);
	FlexGridSizer3->AddGrowableCol(1);
	StaticText4 = new wxStaticText(Panel2, ID_STATICTEXT4, _("Inner Conductor Radius (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT4"));
	FlexGridSizer3->Add(StaticText4, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl4 = new wxTextCtrl(Panel2, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL4"));
	TextCtrl4->SetToolTip(_("Inner conductor radius in metres"));
	FlexGridSizer3->Add(TextCtrl4, 1, wxALL|wxEXPAND, 5);
	StaticText5 = new wxStaticText(Panel2, ID_STATICTEXT5, _("Shield Radius (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT5"));
	FlexGridSizer3->Add(StaticText5, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl5 = new wxTextCtrl(Panel2, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL5"));
	TextCtrl5->SetToolTip(_("Shield radius in metres"));
	FlexGridSizer3->Add(TextCtrl5, 1, wxALL|wxEXPAND, 5);
	StaticText6 = new wxStaticText(Panel2, ID_STATICTEXT6, _("Outer Dielectric Radius (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT6"));
	FlexGridSizer3->Add(StaticText6, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl6 = new wxTextCtrl(Panel2, ID_TEXTCTRL6, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL6"));
	TextCtrl6->SetToolTip(_("Outer dielectric radius in metres"));
	FlexGridSizer3->Add(TextCtrl6, 1, wxALL|wxEXPAND, 5);
	StaticText7 = new wxStaticText(Panel2, ID_STATICTEXT7, _("Inner Conductor Conductivity (S/m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT7"));
	FlexGridSizer3->Add(StaticText7, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl7 = new wxTextCtrl(Panel2, ID_TEXTCTRL7, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL7"));
	TextCtrl7->SetToolTip(_("Inner conductor conductivity in S/m"));
	FlexGridSizer3->Add(TextCtrl7, 1, wxALL|wxEXPAND, 5);
	StaticText8 = new wxStaticText(Panel2, ID_STATICTEXT8, _("Shield Conductor Thickness (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT8"));
	FlexGridSizer3->Add(StaticText8, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl8 = new wxTextCtrl(Panel2, ID_TEXTCTRL8, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL8"));
	TextCtrl8->SetToolTip(_("Shield conductor thickness in metres"));
	FlexGridSizer3->Add(TextCtrl8, 1, wxALL|wxEXPAND, 5);
	StaticText9 = new wxStaticText(Panel2, ID_STATICTEXT9, _("Shield Conductor Conductivity (S/m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT9"));
	FlexGridSizer3->Add(StaticText9, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl9 = new wxTextCtrl(Panel2, ID_TEXTCTRL9, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL9"));
	TextCtrl9->SetToolTip(_("Shield conductor conductivity in S/m"));
	FlexGridSizer3->Add(TextCtrl9, 1, wxALL|wxEXPAND, 5);
	Panel2->SetSizer(FlexGridSizer3);
	FlexGridSizer3->Fit(Panel2);
	FlexGridSizer3->SetSizeHints(Panel2);
	Panel3 = new wxPanel(Notebook1, ID_PANEL3, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANEL3"));
	FlexGridSizer4 = new wxFlexGridSizer(0, 2, 0, 0);
	FlexGridSizer4->AddGrowableCol(0);
	StaticText2 = new wxStaticText(Panel3, ID_STATICTEXT2, _("Inner Dielectric Permittivity"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT2"));
	FlexGridSizer4->Add(StaticText2, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl2 = new wxTextCtrl(Panel3, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL2"));
	TextCtrl2->SetToolTip(_("Inner dielectric permittivity"));
	FlexGridSizer4->Add(TextCtrl2, 1, wxALL|wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL, 5);
	StaticText3 = new wxStaticText(Panel3, ID_STATICTEXT3, _("Outer Dielectric Permittivity"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT3"));
	FlexGridSizer4->Add(StaticText3, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl3 = new wxTextCtrl(Panel3, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL3"));
	TextCtrl3->SetToolTip(_("Outer dielectric permittivity"));
	FlexGridSizer4->Add(TextCtrl3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText10 = new wxStaticText(Panel3, ID_STATICTEXT10, _("Or"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT10"));
	FlexGridSizer4->Add(StaticText10, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText11 = new wxStaticText(Panel3, ID_STATICTEXT11, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT11"));
	FlexGridSizer4->Add(StaticText11, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText12 = new wxStaticText(Panel3, ID_STATICTEXT12, _("Frequency Dependant Model"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT12"));
	FlexGridSizer4->Add(StaticText12, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	CheckBox1 = new wxCheckBox(Panel3, ID_CHECKBOX1, _("Yes"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_CHECKBOX1"));
	CheckBox1->SetValue(false);
	FlexGridSizer4->Add(CheckBox1, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	Panel3->SetSizer(FlexGridSizer4);
	FlexGridSizer4->Fit(Panel3);
	FlexGridSizer4->SetSizeHints(Panel3);
	Panel4 = new wxPanel(Notebook1, ID_PANEL4, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANEL4"));
	FlexGridSizer5 = new wxFlexGridSizer(0, 1, 0, 0);
	FlexGridSizer5->AddGrowableCol(0);
	FlexGridSizer7 = new wxFlexGridSizer(0, 2, 0, 0);
	StaticText18 = new wxStaticText(Panel4, ID_STATICTEXT18, _("R + j w L"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT18"));
	FlexGridSizer7->Add(StaticText18, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	StaticText19 = new wxStaticText(Panel4, ID_STATICTEXT19, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT19"));
	FlexGridSizer7->Add(StaticText19, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText20 = new wxStaticText(Panel4, ID_STATICTEXT20, _("R (ohms/m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT20"));
	FlexGridSizer7->Add(StaticText20, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl15 = new wxTextCtrl(Panel4, ID_TEXTCTRL15, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL15"));
	FlexGridSizer7->Add(TextCtrl15, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText21 = new wxStaticText(Panel4, ID_STATICTEXT21, _("L (henries/m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT21"));
	FlexGridSizer7->Add(StaticText21, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl16 = new wxTextCtrl(Panel4, ID_TEXTCTRL16, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL16"));
	FlexGridSizer7->Add(TextCtrl16, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	CheckBox2 = new wxCheckBox(Panel4, ID_CHECKBOX2, _("Advanced Description"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_CHECKBOX2"));
	CheckBox2->SetValue(false);
	FlexGridSizer7->Add(CheckBox2, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer5->Add(FlexGridSizer7, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer6 = new wxFlexGridSizer(0, 2, 0, 0);
	StaticText15 = new wxStaticText(Panel4, ID_STATICTEXT15, _("Angular f Normalisation"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT15"));
	FlexGridSizer6->Add(StaticText15, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl11 = new wxTextCtrl(Panel4, ID_TEXTCTRL11, _("1"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL11"));
	TextCtrl11->Disable();
	FlexGridSizer6->Add(TextCtrl11, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticLine1 = new wxStaticLine(Panel4, ID_STATICLINE1, wxDefaultPosition, wxSize(10,-1), wxLI_HORIZONTAL, _T("ID_STATICLINE1"));
	FlexGridSizer6->Add(StaticLine1, 1, wxALL|wxEXPAND, 5);
	StaticLine2 = new wxStaticLine(Panel4, ID_STATICLINE2, wxDefaultPosition, wxSize(10,-1), wxLI_HORIZONTAL, _T("ID_STATICLINE2"));
	FlexGridSizer6->Add(StaticLine2, 1, wxALL|wxEXPAND, 5);
	StaticText13 = new wxStaticText(Panel4, ID_STATICTEXT13, _("Order of Numerator Model"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT13"));
	FlexGridSizer6->Add(StaticText13, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl10 = new wxTextCtrl(Panel4, ID_TEXTCTRL10, _("1"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL10"));
	TextCtrl10->Disable();
	FlexGridSizer6->Add(TextCtrl10, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText14 = new wxStaticText(Panel4, ID_STATICTEXT14, _("Numerator Coefficients"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT14"));
	FlexGridSizer6->Add(StaticText14, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl12 = new wxTextCtrl(Panel4, ID_TEXTCTRL12, _("R L"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL12"));
	TextCtrl12->Disable();
	FlexGridSizer6->Add(TextCtrl12, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticLine3 = new wxStaticLine(Panel4, ID_STATICLINE3, wxDefaultPosition, wxSize(10,-1), wxLI_HORIZONTAL, _T("ID_STATICLINE3"));
	FlexGridSizer6->Add(StaticLine3, 1, wxALL|wxEXPAND, 5);
	StaticLine4 = new wxStaticLine(Panel4, ID_STATICLINE4, wxDefaultPosition, wxSize(10,-1), wxLI_HORIZONTAL, _T("ID_STATICLINE4"));
	FlexGridSizer6->Add(StaticLine4, 1, wxALL|wxEXPAND, 5);
	StaticText16 = new wxStaticText(Panel4, ID_STATICTEXT16, _("Order of Denominator Model"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT16"));
	FlexGridSizer6->Add(StaticText16, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl13 = new wxTextCtrl(Panel4, ID_TEXTCTRL13, _("0"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL13"));
	TextCtrl13->Disable();
	FlexGridSizer6->Add(TextCtrl13, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText17 = new wxStaticText(Panel4, ID_STATICTEXT17, _("Denominator Coefficients"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT17"));
	FlexGridSizer6->Add(StaticText17, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl14 = new wxTextCtrl(Panel4, ID_TEXTCTRL14, _("1"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL14"));
	TextCtrl14->Disable();
	FlexGridSizer6->Add(TextCtrl14, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer5->Add(FlexGridSizer6, 1, wxALL|wxEXPAND, 5);
	Panel4->SetSizer(FlexGridSizer5);
	FlexGridSizer5->Fit(Panel4);
	FlexGridSizer5->SetSizeHints(Panel4);
	Notebook1->AddPage(Panel2, _("General"), false);
	Notebook1->AddPage(Panel3, _("Material"), false);
	Notebook1->AddPage(Panel4, _("Transfer Impedance"), false);
	BoxSizer3->Add(Notebook1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(BoxSizer3, 1, wxALL|wxEXPAND, 5);
	StaticBoxSizer1->Add(FlexGridSizer2, 5, wxALL, 5);
	BoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
	Button1 = new wxButton(this, ID_BUTTON1, _("SAVE"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	BoxSizer1->Add(Button1, 1, wxALL|wxALIGN_BOTTOM, 5);
	Button2 = new wxButton(this, ID_BUTTON2, _("BUILD"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
	BoxSizer1->Add(Button2, 1, wxALL|wxALIGN_BOTTOM, 5);
	coaxCLOSE = new wxButton(this, ID_coaxCLOSE, _("CLOSE"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_coaxCLOSE"));
	BoxSizer1->Add(coaxCLOSE, 1, wxALL|wxALIGN_BOTTOM, 5);
	StaticBoxSizer1->Add(BoxSizer1, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer1->Add(StaticBoxSizer1, 1, wxALL|wxEXPAND, 2);
	StaticBoxSizer2 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("Cross Section"));
	Panel1 = new wxPanel(this, ID_PANEL1, wxDefaultPosition, wxSize(500,500), wxTAB_TRAVERSAL, _T("ID_PANEL1"));
	Panel1->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
	Panel1->SetToolTip(_("Double Click to Reset"));
	StaticBoxSizer2->Add(Panel1, 1, wxALL|wxALIGN_TOP, 5);
	FlexGridSizer1->Add(StaticBoxSizer2, 1, wxALL, 2);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);

	Connect(ID_TEXTCTRL4,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl4Text);
	Connect(ID_TEXTCTRL5,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl5Text);
	Connect(ID_TEXTCTRL6,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl6Text);
	Connect(ID_TEXTCTRL7,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl7Text);
	Connect(ID_TEXTCTRL8,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl8Text);
	Connect(ID_TEXTCTRL9,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl9Text);
	Connect(ID_TEXTCTRL2,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl2Text);
	Connect(ID_TEXTCTRL3,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl3Text);
	Connect(ID_CHECKBOX1,wxEVT_COMMAND_CHECKBOX_CLICKED,(wxObjectEventFunction)&COAXIAL::OnCheckBox1Click);
	Connect(ID_TEXTCTRL15,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl15Text);
	Connect(ID_TEXTCTRL16,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl16Text);
	Connect(ID_CHECKBOX2,wxEVT_COMMAND_CHECKBOX_CLICKED,(wxObjectEventFunction)&COAXIAL::OnCheckBox2Click);
	Connect(ID_TEXTCTRL11,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl11Text);
	Connect(ID_TEXTCTRL10,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl10Text);
	Connect(ID_TEXTCTRL12,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl12Text);
	Connect(ID_TEXTCTRL13,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl13Text);
	Connect(ID_TEXTCTRL14,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&COAXIAL::OnTextCtrl14Text);
	Connect(ID_NOTEBOOK1,wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING,(wxObjectEventFunction)&COAXIAL::OnNotebook1PageChanging);
	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&COAXIAL::OnButton1Click);
	Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&COAXIAL::OnButton2Click);
	Connect(ID_coaxCLOSE,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&COAXIAL::OncoaxCLOSEClick);
	Panel1->Connect(wxEVT_PAINT,(wxObjectEventFunction)&COAXIAL::OnPanel1Paint,0,this);
	Panel1->Connect(wxEVT_LEFT_DCLICK,(wxObjectEventFunction)&COAXIAL::OnPanel1LeftDClick,0,this);
	//*)

    TextCtrl2->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    TextCtrl3->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    TextCtrl4->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    TextCtrl5->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    TextCtrl6->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    TextCtrl7->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    TextCtrl8->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    TextCtrl9->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
    CheckBox1->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&COAXIAL::OnHighlightSchematic, this);
}

COAXIAL::~COAXIAL()
{
	//(*Destroy(COAXIAL)
	//*)
}

void COAXIAL::SetPathToMOD(wxString Path, wxString Name)
{
    pathToMOD << Path;
    MODname << Name;
}

void COAXIAL::OnButton1Click(wxCommandEvent& event)
{
    int IsError = 0;

    wxString fileName = TextCtrl1->GetValue();

    TextCtrl1->SetBackgroundColour(wxColor(255,255,255));

    if (fileName == "")
    {
        wxMessageBox("Please Specify a Cable Name" , _("Missing Cable Name"));
        return;
    }


    check_is_empty(TextCtrl4, &IsError);
    check_is_empty(TextCtrl5, &IsError);
    check_is_empty(TextCtrl6, &IsError);
    check_is_empty(TextCtrl7, &IsError);
    check_is_empty(TextCtrl8, &IsError);
    check_is_empty(TextCtrl9, &IsError);
    check_is_empty(TextCtrl10, &IsError);
    check_is_empty(TextCtrl11, &IsError);
    check_is_empty(TextCtrl12, &IsError);
    check_is_empty(TextCtrl13, &IsError);
    check_is_empty(TextCtrl14, &IsError);

    if (CheckBox1->GetValue())
    {

    }
    else
    {
        check_is_empty(TextCtrl2, &IsError);
        check_is_empty(TextCtrl3, &IsError);

    }

    if ( IsError == 1)
    {
        wxMessageBox("Please fix highlighted errors" , _("Non numeric entry"));
        return;
    }

    WriteCableFile(fileName);

    Button1->Disable();
}

void COAXIAL::OnButton2Click(wxCommandEvent& event)
{
    wxString fileName = TextCtrl1->GetValue();
    wxString fullFileName;

    fullFileName = pathToMOD + "\\CABLE\\" + fileName;

    std::stringstream stream;

    stream  << "\"..\\BIN\\cable_model_builder.exe\""
            << " "
            << "..\\" + MODname + "\\CABLE\\"
            << fileName ;

    system(stream.str().c_str());

    // Display Run Status
    RUN_STATUS *run_status = new RUN_STATUS(this);
    run_status->ShowModal();

    Button1->Enable();
}

void COAXIAL::OncoaxCLOSEClick(wxCommandEvent& event)
{
    //clean up directory
    clean_temp_files();
    Close();
}

void COAXIAL::OnCheckBox1Click(wxCommandEvent& event)
{
    TextCtrl2->Disable();
    TextCtrl3->Disable();

    FD_ESR *winFD_ESR = new FD_ESR(this);

    winFD_ESR->formtype = 2;

    if ( winFD_ESR->ShowModal() == wxID_OK )
    {
        FD_ESR1 = winFD_ESR->TextCtrl1->GetValue();
        FD_ESR2 = winFD_ESR->TextCtrl2->GetValue();
        FD_ESR3 = winFD_ESR->TextCtrl3->GetValue();
        FD_ESR4 = winFD_ESR->TextCtrl4->GetValue();
        FD_ESR5 = winFD_ESR->TextCtrl5->GetValue();

        FD_ESR6 = winFD_ESR->TextCtrl6->GetValue();
        FD_ESR7 = winFD_ESR->TextCtrl7->GetValue();
        FD_ESR8 = winFD_ESR->TextCtrl8->GetValue();
        FD_ESR9 = winFD_ESR->TextCtrl9->GetValue();
        FD_ESR10= winFD_ESR->TextCtrl10->GetValue();
    }
    winFD_ESR->Destroy();
}

void COAXIAL::OnTextCtrl1Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_filename_valid(TextCtrl1, &IsError);
}

void COAXIAL::OnTextCtrl2Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl2, &IsError);
    check_is_positive(TextCtrl2, &IsError);
}

void COAXIAL::OnTextCtrl3Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl3, &IsError);
    check_is_positive(TextCtrl3, &IsError);
}

void COAXIAL::OnTextCtrl4Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl4, &IsError);
    check_is_positive(TextCtrl4, &IsError);
}

void COAXIAL::OnTextCtrl5Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl5, &IsError);
    check_is_positive(TextCtrl5, &IsError);
    check_is_less_than(TextCtrl5, TextCtrl4, &IsError);
}

void COAXIAL::OnTextCtrl6Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl6, &IsError);
    check_is_positive(TextCtrl6, &IsError);
}

void COAXIAL::OnTextCtrl7Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl7, &IsError);
    check_is_positive(TextCtrl7, &IsError);
}

void COAXIAL::OnTextCtrl8Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl8, &IsError);
    check_is_positive(TextCtrl8, &IsError);
}

void COAXIAL::OnTextCtrl9Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl9, &IsError);
    check_is_positive(TextCtrl9, &IsError);
}

void COAXIAL::OnTextCtrl10Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl10, &IsError);
    check_is_positive(TextCtrl10, &IsError);
}

void COAXIAL::OnTextCtrl11Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl11, &IsError);
}

void COAXIAL::OnTextCtrl12Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl12, &IsError);
}

void COAXIAL::OnTextCtrl13Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl13, &IsError);
    check_is_positive(TextCtrl13, &IsError);
}

void COAXIAL::OnTextCtrl14Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl14, &IsError);
}

void COAXIAL::OnTextCtrl15Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl15, &IsError);
    check_is_positive(TextCtrl15, &IsError);
}

void COAXIAL::OnTextCtrl16Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl15, &IsError);
    check_is_positive(TextCtrl15, &IsError);
}


void COAXIAL::WriteCableFile(wxString fileName)
{
    ofstream cableFile;

    wxString fullFileName = pathToMOD + "//CABLE//" + fileName + ".cable_spec";

    cableFile.open (fullFileName);

    if (cableFile.is_open())
    {

        // minimum requirements for type Coax
        cableFile <<"# MOD_cable_lib_dir \n";
        cableFile <<"../"<< MODname <<"/CABLE/"<<"\n";
        cableFile <<"Coax\n";
        cableFile <<"2 \t#number of conductors \n";
        cableFile <<"6 \t#number of parameters \n";
        cableFile << TextCtrl4->GetValue() <<"\t # parameter 1: inner conductor radius (m)\n";
        cableFile << TextCtrl5->GetValue() <<"\t # parameter 2: shield radius\n";
        cableFile << TextCtrl6->GetValue() <<"\t # parameter 3: outer insulation radius (m)\n";
        cableFile << TextCtrl7->GetValue() <<"\t # parameter 4: inner conductor electric conductivity\n";
        cableFile << TextCtrl8->GetValue() <<"\t # parameter 5: shield conductor thickness\n";
        cableFile << TextCtrl9->GetValue() <<" \t# parameter 6: shield electric conductivity\n";
        cableFile <<"2 \t# number of frequency dependent parameters\n";;


        if (CheckBox1->GetValue()){
            cableFile << "# Inner dielectric relative permittivity\n";
            cableFile << FD_ESR1 <<"\t # w normalisation constant\n";
            cableFile << FD_ESR2 <<"\t # a order, a coefficients follow below\n";
            cableFile << FD_ESR3 <<"\n";
            cableFile << FD_ESR4 <<"\t # b order, b coefficients follow below\n";
            cableFile << FD_ESR5 <<"\n";

            cableFile << "#  Outer dielectric relative permittivity\n";
            cableFile << FD_ESR6 <<"\t # w normalisation constant\n";
            cableFile << FD_ESR7 <<"\t # a order, a coefficients follow below\n";
            cableFile << FD_ESR8 <<"\n";
            cableFile << FD_ESR9 <<"\t # b order, b coefficients follow below\n";
            cableFile << FD_ESR10<<"\n";

        }

        else{
            cableFile << "# Inner dielectric relative permittivity\n";
            cableFile << "1.0 # w normalisation constant\n";
            cableFile << "0 # a order, a coefficients follow below\n";
            cableFile << TextCtrl2->GetValue()<<"\n";
            cableFile << "0 # b order, b coefficients follow below\n";
            cableFile << "1.0\n";

            cableFile << "#  Outer dielectric relative permittivity\n";
            cableFile << "1.0 \t# w normalisation constant\n";
            cableFile << "0 \t# a order, a coefficients follow below\n";
            cableFile << TextCtrl3->GetValue()<<"\n";
            cableFile << "0 \t# b order, b coefficients follow below\n";
            cableFile << "1.0\n";
        }

        cableFile << "1 \t# number of frequency dependent transfer impedance models\n";
        cableFile << "#  Transfer impedance model \n";

        if (CheckBox2->GetValue())
        {
            cableFile << TextCtrl11->GetValue() <<" \t# angular frequency normalisation\n";
            cableFile << TextCtrl10->GetValue() <<" \t# order of numerator model\n";
            cableFile << TextCtrl12->GetValue() <<" \t# list of numerator coefficients a0 a1 a2... \n";
            cableFile << TextCtrl13->GetValue() <<" \t# order of denominator model\n";
            cableFile << TextCtrl14->GetValue() <<" \t# list of denominator coefficients b0 b1 b2... \n";
        }

        else
        {
            cableFile << "1 \t# angular frequency normalisation\n";
            cableFile << "1 \t# order of numerator model\n";
            cableFile << TextCtrl15->GetValue() << " ";
            cableFile << TextCtrl16->GetValue() << " \t# list of numerator coefficients a0 a1 a2... \n";
            cableFile << "0 \t# order of denominator model\n";
            cableFile << "1 \t# list of denominator coefficients b0 b1 b2... \n";
        }

    }
    cableFile.close();
}

void COAXIAL::OnPanel1Paint(wxPaintEvent& event)
{
    coaxialSchematic();
}

void COAXIAL::coaxialSchematic()
{
    int size_h, size_v;
    Panel1->GetSize(&size_h, &size_v );

    wxClientDC dc(Panel1);
    dc.Clear();

    // Draw centre lines
    dc.SetPen( wxPen( wxColor(224,224,224),1 , wxDOT_DASH) );
    dc.DrawLine( 0, size_v/2, size_h,size_v/2 );
    dc.DrawLine( size_h/2, 0, size_h/2, size_v);

    dc.SetTextForeground(*wxBLUE);
    dc.DrawText("General Coaxial Cable", 10, 15);

    float conductor_radius, dielectric_radius, shield_radius, insulation_radius;

    conductor_radius  = 0.125 * size_h;
    dielectric_radius = 0.25 * size_h;
    shield_radius     = 0.26 * size_h;
    insulation_radius = 0.3 * size_h;

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxBDIAGONAL_HATCH));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2), insulation_radius );

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2), shield_radius );

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(*wxWHITE, wxSOLID));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxFDIAGONAL_HATCH));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius);
}

void COAXIAL::OnHighlightSchematic(wxCommandEvent& event)
{
    coaxialSchematic();

    int ID = event.GetId();
    event.Skip(true);

    int size_h, size_v;
    Panel1->GetSize(&size_h, &size_v );

    float conductor_radius, dielectric_radius, shield_radius, insulation_radius;

    conductor_radius  = 0.125 * size_h;
    dielectric_radius = 0.25 * size_h;
    shield_radius     = 0.26 * size_h;
    insulation_radius = 0.3*size_h;

    float x, y;

    wxClientDC dc(Panel1);

    wxPoint arrowHead[3];

    if ( ID == ID_TEXTCTRL4)
    {
        x = conductor_radius * 0.7071;
        y = conductor_radius * 0.7071;
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2),4 );
        dc.DrawLine( size_h/2,size_v/2, size_h/2+x, size_v/2-y );

        arrowHead[0] = wxPoint(size_h/2+x,size_v/2-y);
        arrowHead[1] = wxPoint(size_h/2+x-5,size_v/2-y);
        arrowHead[2] = wxPoint(size_h/2+x,size_v/2-y+5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);
    }

    if ( ID == ID_TEXTCTRL5)
    {
        x = dielectric_radius * 0.7071;
        y = dielectric_radius * 0.7071;
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2),4 );
        dc.DrawLine( size_h/2,size_v/2, size_h/2+x, size_v/2+y );

        arrowHead[0] = wxPoint(size_h/2+x,size_v/2+y);
        arrowHead[1] = wxPoint(size_h/2+x-5,size_v/2+y);
        arrowHead[2] = wxPoint(size_h/2+x,size_v/2+y-5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);

    }

    if ( ID == ID_TEXTCTRL6)
    {
        x = insulation_radius * 0.7071;
        y = insulation_radius * 0.7071;
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2),4 );
        dc.DrawLine( size_h/2,size_v/2, size_h/2-x, size_v/2-y );

        arrowHead[0] = wxPoint(size_h/2-x,size_v/2-y);
        arrowHead[1] = wxPoint(size_h/2-x+5,size_v/2-y);
        arrowHead[2] = wxPoint(size_h/2-x,size_v/2-y+5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);
    }

    if ( ID == ID_TEXTCTRL7)
    {
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius );
    }

    if ( ID == ID_TEXTCTRL8)
    {
        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), shield_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(*wxWHITE, wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxFDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius);

    }

    if ( ID == ID_TEXTCTRL9)
    {
        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), shield_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(*wxWHITE, wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxFDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius);
    }

    if ( ID == ID_TEXTCTRL2 || ID == ID_CHECKBOX1 )
    {
        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(*wxWHITE, wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

        dc.SetBrush(wxBrush(wxColor(51,51,255), wxFDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius);
    }

    if ( ID == ID_TEXTCTRL3 || ID == ID_CHECKBOX1 )
    {
        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxBDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), insulation_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), shield_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(*wxWHITE, wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxFDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius);
    }

    if ( ID == ID_CHECKBOX1)
    {
        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxBDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), insulation_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), shield_radius );

        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(*wxWHITE, wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxFDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius);
    }
}


void COAXIAL::OnPanel1LeftDClick(wxMouseEvent& event)
{
    coaxialSchematic();
}

void COAXIAL::OnNotebook1PageChanging(wxNotebookEvent& event)
{
    coaxialSchematic();
}

void COAXIAL::OnCheckBox2Click(wxCommandEvent& event)
{
    if (CheckBox2->GetValue())
    {
        TextCtrl15->Disable();
        TextCtrl16->Disable();
        TextCtrl10->Enable();
        TextCtrl11->Enable();
        TextCtrl12->Enable();
        TextCtrl13->Enable();
        TextCtrl14->Enable();
    }
    else
    {
        TextCtrl15->Enable();
        TextCtrl16->Enable();
        TextCtrl10->Disable();
        TextCtrl11->Disable();
        TextCtrl12->Disable();
        TextCtrl13->Disable();
        TextCtrl14->Disable();
    }
}


