/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     CYLINDRICAL.cpp
//
// DESCRIPTION
//     GUI for the construction of cable_spec file for cable type: Cylindrical
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////

#include "CYLINDRICAL.h"

//(*InternalHeaders(CYLINDRICAL)
#include <wx/settings.h>
#include <wx/intl.h>
#include <wx/string.h>
//*)

//(*IdInit(CYLINDRICAL)
const long CYLINDRICAL::ID_STATICTEXT1 = wxNewId();
const long CYLINDRICAL::ID_TEXTCTRL1 = wxNewId();
const long CYLINDRICAL::ID_STATICTEXT2 = wxNewId();
const long CYLINDRICAL::ID_TEXTCTRL2 = wxNewId();
const long CYLINDRICAL::ID_STATICTEXT3 = wxNewId();
const long CYLINDRICAL::ID_TEXTCTRL3 = wxNewId();
const long CYLINDRICAL::ID_STATICTEXT4 = wxNewId();
const long CYLINDRICAL::ID_TEXTCTRL4 = wxNewId();
const long CYLINDRICAL::ID_STATICTEXT5 = wxNewId();
const long CYLINDRICAL::ID_TEXTCTRL5 = wxNewId();
const long CYLINDRICAL::ID_STATICTEXT6 = wxNewId();
const long CYLINDRICAL::ID_STATICTEXT7 = wxNewId();
const long CYLINDRICAL::ID_STATICTEXT8 = wxNewId();
const long CYLINDRICAL::ID_CHECKBOX1 = wxNewId();
const long CYLINDRICAL::ID_BUTTON1 = wxNewId();
const long CYLINDRICAL::ID_BUTTON2 = wxNewId();
const long CYLINDRICAL::ID_BUTTON3 = wxNewId();
const long CYLINDRICAL::ID_PANEL1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(CYLINDRICAL,wxDialog)
	//(*EventTable(CYLINDRICAL)
	//*)
END_EVENT_TABLE()

CYLINDRICAL::CYLINDRICAL(wxWindow* parent,wxWindowID id,const wxPoint& pos,const wxSize& size)
{
	//(*Initialize(CYLINDRICAL)
	wxStaticBoxSizer* StaticBoxSizer2;
	wxFlexGridSizer* FlexGridSizer4;
	wxStaticBoxSizer* StaticBoxSizer4;
	wxFlexGridSizer* FlexGridSizer3;
	wxFlexGridSizer* FlexGridSizer2;
	wxBoxSizer* BoxSizer2;
	wxStaticBoxSizer* StaticBoxSizer3;
	wxBoxSizer* BoxSizer1;
	wxStaticBoxSizer* StaticBoxSizer1;
	wxFlexGridSizer* FlexGridSizer1;

	Create(parent, wxID_ANY, _("SACAMOS: Cylindrical Cable Model"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxMAXIMIZE_BOX|wxMINIMIZE_BOX, _T("wxID_ANY"));
	FlexGridSizer1 = new wxFlexGridSizer(2, 2, 0, 5);
	FlexGridSizer1->AddGrowableCol(1);
	StaticBoxSizer1 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("Cyindrical Cable Parameters"));
	FlexGridSizer2 = new wxFlexGridSizer(0, 1, 0, 0);
	FlexGridSizer2->AddGrowableCol(0);
	BoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
	StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("Cable Name"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
	BoxSizer1->Add(StaticText1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl1 = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	BoxSizer1->Add(TextCtrl1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(BoxSizer1, 1, wxALL|wxEXPAND, 0);
	StaticBoxSizer3 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("General Properties"));
	FlexGridSizer3 = new wxFlexGridSizer(0, 2, 0, 1);
	FlexGridSizer3->AddGrowableCol(0);
	StaticText2 = new wxStaticText(this, ID_STATICTEXT2, _("Conductor Radius (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT2"));
	FlexGridSizer3->Add(StaticText2, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl2 = new wxTextCtrl(this, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL2"));
	FlexGridSizer3->Add(TextCtrl2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText3 = new wxStaticText(this, ID_STATICTEXT3, _("Dielectric Radius (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT3"));
	FlexGridSizer3->Add(StaticText3, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl3 = new wxTextCtrl(this, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL3"));
	FlexGridSizer3->Add(TextCtrl3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText4 = new wxStaticText(this, ID_STATICTEXT4, _("Conductor Conductivity (S/m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT4"));
	FlexGridSizer3->Add(StaticText4, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl4 = new wxTextCtrl(this, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL4"));
	FlexGridSizer3->Add(TextCtrl4, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticBoxSizer3->Add(FlexGridSizer3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(StaticBoxSizer3, 1, wxALL|wxEXPAND, 5);
	StaticBoxSizer4 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("Dielectric Permittivity Model"));
	FlexGridSizer4 = new wxFlexGridSizer(0, 2, 0, 0);
	FlexGridSizer4->AddGrowableCol(0);
	StaticText5 = new wxStaticText(this, ID_STATICTEXT5, _("Dielectric Permittivity"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT5"));
	FlexGridSizer4->Add(StaticText5, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl5 = new wxTextCtrl(this, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL5"));
	FlexGridSizer4->Add(TextCtrl5, 1, wxALL|wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL, 5);
	StaticText6 = new wxStaticText(this, ID_STATICTEXT6, _("Or"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT6"));
	FlexGridSizer4->Add(StaticText6, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText7 = new wxStaticText(this, ID_STATICTEXT7, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT7"));
	FlexGridSizer4->Add(StaticText7, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText8 = new wxStaticText(this, ID_STATICTEXT8, _("Frequency Dependent Model"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT8"));
	FlexGridSizer4->Add(StaticText8, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	CheckBox1 = new wxCheckBox(this, ID_CHECKBOX1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_CHECKBOX1"));
	CheckBox1->SetValue(false);
	FlexGridSizer4->Add(CheckBox1, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	StaticBoxSizer4->Add(FlexGridSizer4, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer2->Add(StaticBoxSizer4, 1, wxALL|wxEXPAND, 5);
	BoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
	Button1 = new wxButton(this, ID_BUTTON1, _("SAVE"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	BoxSizer2->Add(Button1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	Button2 = new wxButton(this, ID_BUTTON2, _("BUILD"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
	BoxSizer2->Add(Button2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	Button3 = new wxButton(this, ID_BUTTON3, _("CLOSE"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
	BoxSizer2->Add(Button3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(BoxSizer2, 1, wxALL|wxEXPAND, 5);
	StaticBoxSizer1->Add(FlexGridSizer2, 1, wxALL, 2);
	FlexGridSizer1->Add(StaticBoxSizer1, 1, wxALL|wxALIGN_TOP|wxALIGN_CENTER_HORIZONTAL, 5);
	StaticBoxSizer2 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("Cross Section"));
	Panel1 = new wxPanel(this, ID_PANEL1, wxDefaultPosition, wxSize(500,500), wxTAB_TRAVERSAL, _T("ID_PANEL1"));
	Panel1->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
	Panel1->SetToolTip(_("Double Click to Reset Schematic"));
	StaticBoxSizer2->Add(Panel1, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer1->Add(StaticBoxSizer2, 1, wxALL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);

	Connect(ID_TEXTCTRL1,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&CYLINDRICAL::OnTextCtrl1Text);
	Connect(ID_TEXTCTRL2,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&CYLINDRICAL::OnTextCtrl2Text);
	Connect(ID_TEXTCTRL3,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&CYLINDRICAL::OnTextCtrl3Text);
	Connect(ID_TEXTCTRL4,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&CYLINDRICAL::OnTextCtrl4Text);
	Connect(ID_TEXTCTRL5,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&CYLINDRICAL::OnTextCtrl5Text);
	Connect(ID_CHECKBOX1,wxEVT_COMMAND_CHECKBOX_CLICKED,(wxObjectEventFunction)&CYLINDRICAL::OnCheckBox1Click);
	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&CYLINDRICAL::OnButton1Click);
	Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&CYLINDRICAL::OnButton2Click);
	Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&CYLINDRICAL::OnButton3Click);
	Panel1->Connect(wxEVT_PAINT,(wxObjectEventFunction)&CYLINDRICAL::OnPanel1Paint,0,this);
	Panel1->Connect(wxEVT_LEFT_DCLICK,(wxObjectEventFunction)&CYLINDRICAL::OnPanel1LeftDClick,0,this);
	//*)

    TextCtrl2->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&CYLINDRICAL::OnHighlightSchematic, this);
    TextCtrl3->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&CYLINDRICAL::OnHighlightSchematic, this);
    TextCtrl4->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&CYLINDRICAL::OnHighlightSchematic, this);
    TextCtrl5->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&CYLINDRICAL::OnHighlightSchematic, this);
    CheckBox1->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&CYLINDRICAL::OnHighlightSchematic, this);


}

CYLINDRICAL::~CYLINDRICAL()
{
    //(*Destroy(CYLINDRICAL)
	//*)

}

void CYLINDRICAL::SetPathToMOD(wxString Path, wxString Name)
{
    pathToMOD << Path;
    MODname << Name;
}

void CYLINDRICAL::OnButton1Click(wxCommandEvent& event)
{
    int IsError = 0;

    wxString fileName = TextCtrl1->GetValue();

    TextCtrl1->SetBackgroundColour(wxColor(255,255,255));

    if (fileName == "")
    {
        wxMessageBox("Please Specify a Cable Name" , _("Missing Cable Name"));
        return;
    }

    check_is_empty(TextCtrl2, &IsError);
    check_is_empty(TextCtrl3, &IsError);
    check_is_empty(TextCtrl4, &IsError);

    if (CheckBox1->GetValue()){
//        check_is_empty(TextCtrl6, &IsError);
//        check_is_empty(TextCtrl7, &IsError);
//        check_is_empty(TextCtrl8, &IsError);
//        check_is_empty(TextCtrl9, &IsError);
//        check_is_empty(TextCtrl10, &IsError);
    }
    else
    {
        check_is_empty(TextCtrl5, &IsError);
    }

    if ( IsError == 1)
    {
        wxMessageBox("Please fix highlighted errors" , _("Non numeric entry"));
        return;
    }
    WriteCableFile (fileName);

    Button1->Disable();
}

void CYLINDRICAL::OnButton2Click(wxCommandEvent& event)
{

    wxString fileName = TextCtrl1->GetValue();
    wxString fineName = fileName;

    std::stringstream stream;
    stream  << "\"..\\BIN\\cable_model_builder.exe\""
            << " "
            << "..\\" + MODname + "\\CABLE\\"
            << fileName ;
            //<< "..\\MOD\\CABLE\\"   // relative path to MOD


    system(stream.str().c_str());

    // Display Run Status
    RUN_STATUS *run_status = new RUN_STATUS(this);
    run_status->ShowModal();

    Button1->Enable();

}

void CYLINDRICAL::OnButton3Click(wxCommandEvent& event)
{
    //clean up directory
    clean_temp_files();
    Close();
}

void CYLINDRICAL::OnTextCtrl1Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_filename_valid(TextCtrl1, &IsError);
}

void CYLINDRICAL::OnTextCtrl2Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl2, &IsError);
    check_is_positive (TextCtrl2, &IsError);
}

void CYLINDRICAL::OnTextCtrl3Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl3, &IsError);
    check_is_positive (TextCtrl3, &IsError);
    check_is_less_than(TextCtrl3, TextCtrl2, &IsError);
}

void CYLINDRICAL::OnTextCtrl4Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl4, &IsError);
    check_is_positive (TextCtrl4, &IsError);

}

void CYLINDRICAL::OnTextCtrl5Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric(TextCtrl5, &IsError);
    check_is_positive (TextCtrl5, &IsError);
}

//void CYLINDRICAL::OnTextCtrl6Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric(TextCtrl6, &IsError);
//    check_is_positive (TextCtrl6, &IsError);
//}
//
//void CYLINDRICAL::OnTextCtrl7Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric(TextCtrl7, &IsError);
//}
//
//void CYLINDRICAL::OnTextCtrl8Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric_list(TextCtrl8, &IsError);
//}
//
//void CYLINDRICAL::OnTextCtrl9Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric(TextCtrl9, &IsError);
//}
//
//void CYLINDRICAL::OnTextCtrl10Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric_list(TextCtrl10, &IsError);
//}

void CYLINDRICAL::OnCheckBox1Click(wxCommandEvent& event)
{
    if (CheckBox1->GetValue())
    {
        TextCtrl5->Disable();

        FD_ESR *winFD_ESR = new FD_ESR(this);

        winFD_ESR->formtype = 1;

        if ( winFD_ESR->ShowModal() == wxID_OK )
        {
            FD_ESR6 = winFD_ESR->TextCtrl6->GetValue();
            FD_ESR7 = winFD_ESR->TextCtrl7->GetValue();
            FD_ESR8 = winFD_ESR->TextCtrl8->GetValue();
            FD_ESR9 = winFD_ESR->TextCtrl9->GetValue();
            FD_ESR10= winFD_ESR->TextCtrl10->GetValue();
        }
        winFD_ESR->Destroy();
    }
//    if (CheckBox1->GetValue())
//    {
//        TextCtrl5->Disable();
//        TextCtrl6->Enable();
//        TextCtrl7->Enable();
//        TextCtrl8->Enable();
//        TextCtrl9->Enable();
//        TextCtrl10->Enable();
//    }
//    else
//    {
//        TextCtrl5->Enable();
//        TextCtrl6->Disable();
//        TextCtrl7->Disable();
//        TextCtrl8->Disable();
//        TextCtrl9->Disable();
//        TextCtrl10->Disable();
//    }
}

void CYLINDRICAL::WriteCableFile (wxString fileName)
{

    ofstream cableFile;

    wxString fullFileName = pathToMOD + "\\CABLE\\" + fileName + ".cable_spec";

    cableFile.open (fullFileName);

    if (cableFile.is_open())
    {
        // minimum requirements for type Cylindrical
        cableFile <<"# MOD_cable_lib_dir \n";
        cableFile <<"../"<< MODname <<"/CABLE/"<<"\n";
        cableFile <<"Cylindrical\n";
        cableFile <<"1 #number of conductors \n";
        cableFile <<"3 #number of parameters \n";
        cableFile << TextCtrl2->GetValue() <<" # parameter 1: conductor radius\n";
        cableFile << TextCtrl3->GetValue() <<" # parameter 2: dielectric radius\n";
        cableFile << TextCtrl4->GetValue() <<" # parameter 3: conductivity\n";
        cableFile <<"1 \t\t\t# number of frequency dependent parameters\n";

        if (CheckBox1->GetValue()){
//            cableFile << "# Dielectric relative permittivity model follows\n";
//            cableFile << TextCtrl6->GetValue() <<" # w normalisation constant\n";
//            cableFile << TextCtrl7->GetValue() <<" # a order, a coefficients follow below\n";
//            cableFile << TextCtrl8->GetValue() <<" \n";
//            cableFile << TextCtrl9->GetValue() <<" # b order, b coefficients follow below\n";
//            cableFile << TextCtrl10->GetValue() <<" \n";
            cableFile << "# Dielectric relative permittivity model follows\n";
            cableFile << FD_ESR6 <<"\t # w normalisation constant\n";
            cableFile << FD_ESR7 <<"\t # a order, a coefficients follow below\n";
            cableFile << FD_ESR8 <<"\n";
            cableFile << FD_ESR9 <<"\t # b order, b coefficients follow below\n";
            cableFile << FD_ESR10 <<"\n";
        }

        else{
            cableFile << "# Dielectric relative permittivity model follows\n";
            cableFile << "1.0 # w normalisation constant\n";
            cableFile << "0 # a order, a coefficients follow below\n";
            cableFile << TextCtrl5->GetValue()<<"\n";
            cableFile << "0 # b order, b coefficients follow below\n";
            cableFile << "1.0\n";
        }

        cableFile.close();
    }
}

void CYLINDRICAL::OnPanel1Paint(wxPaintEvent& event)
{
    cylindricalSchematic();
}

void CYLINDRICAL::cylindricalSchematic()
{
    int size_h, size_v;
    Panel1->GetSize(&size_h, &size_v );

    wxClientDC dc(Panel1);
    dc.Clear();

    // Draw centre lines
    dc.SetPen( wxPen( wxColor(224,224,224),1 , wxDOT_DASH) );
    dc.DrawLine( 0, size_v/2, size_h,size_v/2 );
    dc.DrawLine( size_h/2, 0, size_h/2, size_v);

    dc.SetTextForeground(*wxBLUE);
    dc.DrawText("General Cylindrical Cable with Dielectric", 10, 15);

    float conductor_radius, dielectric_radius;

    conductor_radius = 0.125 * size_h;
    dielectric_radius = 0.25 * size_h;

    dc.SetPen( wxPen(wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius );

    dc.SetBrush(wxBrush(wxColor(224,224,224), wxBDIAGONAL_HATCH));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );

    dc.SetPen( wxPen(wxColor(192,192,192),1 ) );
}

void CYLINDRICAL::OnHighlightSchematic(wxCommandEvent& event)
{
    cylindricalSchematic();

    int ID = event.GetId();
    event.Skip(true);

    int size_h, size_v;
    Panel1->GetSize(&size_h, &size_v );

    float conductor_radius, dielectric_radius;

    conductor_radius = 0.125 * size_h;
    dielectric_radius = 0.25 * size_h;

    float x, y;

    wxClientDC dc(Panel1);

    wxPoint arrowHead[3];

    if ( ID == ID_TEXTCTRL2)
    {
        x = conductor_radius * 0.7071;
        y = conductor_radius * 0.7071;
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2),4 );
        dc.DrawLine( size_h/2,size_v/2, size_h/2+x, size_v/2-y );

        arrowHead[0] = wxPoint(size_h/2+x,size_v/2-y);
        arrowHead[1] = wxPoint(size_h/2+x-5,size_v/2-y);
        arrowHead[2] = wxPoint(size_h/2+x,size_v/2-y+5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);

    }

    if ( ID == ID_TEXTCTRL3)
    {
        x = dielectric_radius * 0.7071;
        y = dielectric_radius * 0.7071;
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2),4 );
        dc.DrawLine( size_h/2,size_v/2, size_h/2+x, size_v/2+y );

        arrowHead[0] = wxPoint(size_h/2+x,size_v/2+y);
        arrowHead[1] = wxPoint(size_h/2+x-5,size_v/2+y);
        arrowHead[2] = wxPoint(size_h/2+x,size_v/2+y-5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);

    }

    if ( ID == ID_TEXTCTRL4)
    {
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius );
    }

    if ( ID == ID_TEXTCTRL5)
    {
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxBDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxBDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius );

    }
    if ( ID == ID_CHECKBOX1)
    {
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxBDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), dielectric_radius );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxBDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2), conductor_radius );

    }
}

void CYLINDRICAL::OnPanel1LeftDClick(wxMouseEvent& event)
{
    cylindricalSchematic();
}
