/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     CYLINDRICAL.h
//
// DESCRIPTION
//     GUI for the construction of cable_spec file for cable type: Cylindrical
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////
#ifndef CYLINDRICAL_H
#define CYLINDRICAL_H

//(*Headers(CYLINDRICAL)
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/panel.h>
#include <wx/button.h>
#include <wx/dialog.h>
//*)

// Additional wxWidgets Includes
#include <wx/msgdlg.h>
#include <wx/dcclient.h>

// Application Includes
#include "FD_ESR.h"
#include "UTILITIES.h"
#include "RUN_STATUS.h"

// Standard Library Header Files
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

class CYLINDRICAL: public wxDialog
{
	public:

		CYLINDRICAL(wxWindow* parent,wxWindowID id=wxID_ANY,const wxPoint& pos=wxDefaultPosition,const wxSize& size=wxDefaultSize);
		virtual ~CYLINDRICAL();

		void SetPathToMOD(wxString, wxString);
		wxString pathToMOD;
		wxString MODname;

		void WriteCableFile(wxString);

        void cylindricalSchematic();
        void OnHighlightSchematic(wxCommandEvent & event);

        wxString FD_ESR1;
        wxString FD_ESR2;
        wxString FD_ESR3;
        wxString FD_ESR4;
        wxString FD_ESR5;
        wxString FD_ESR6;
        wxString FD_ESR7;
        wxString FD_ESR8;
        wxString FD_ESR9;
        wxString FD_ESR10;

        wxString FD_ORDER;
        bool FD_LINLOG;

        wxString FD_FMIN;
        wxString FD_FMAX;
        wxString FD_NUMF;

        //double a0;

		//(*Declarations(CYLINDRICAL)
		wxTextCtrl* TextCtrl4;
		wxStaticText* StaticText2;
		wxButton* Button1;
		wxStaticText* StaticText6;
		wxStaticText* StaticText8;
		wxPanel* Panel1;
		wxStaticText* StaticText1;
		wxStaticText* StaticText3;
		wxButton* Button2;
		wxCheckBox* CheckBox1;
		wxButton* Button3;
		wxStaticText* StaticText5;
		wxStaticText* StaticText7;
		wxTextCtrl* TextCtrl2;
		wxTextCtrl* TextCtrl1;
		wxTextCtrl* TextCtrl5;
		wxStaticText* StaticText4;
		wxTextCtrl* TextCtrl3;
		//*)

	protected:

		//(*Identifiers(CYLINDRICAL)
		static const long ID_STATICTEXT1;
		static const long ID_TEXTCTRL1;
		static const long ID_STATICTEXT2;
		static const long ID_TEXTCTRL2;
		static const long ID_STATICTEXT3;
		static const long ID_TEXTCTRL3;
		static const long ID_STATICTEXT4;
		static const long ID_TEXTCTRL4;
		static const long ID_STATICTEXT5;
		static const long ID_TEXTCTRL5;
		static const long ID_STATICTEXT6;
		static const long ID_STATICTEXT7;
		static const long ID_STATICTEXT8;
		static const long ID_CHECKBOX1;
		static const long ID_BUTTON1;
		static const long ID_BUTTON2;
		static const long ID_BUTTON3;
		static const long ID_PANEL1;
		//*)

	private:

		//(*Handlers(CYLINDRICAL)
		void OnButton2Click(wxCommandEvent& event);
		void OnButton3Click(wxCommandEvent& event);
		void OnButton1Click(wxCommandEvent& event);
		void OnCheckBox1Click(wxCommandEvent& event);
		void OnTextCtrl2Text(wxCommandEvent& event);
		void OnTextCtrl2TextEnter(wxCommandEvent& event);
		void OnPanel1Paint(wxPaintEvent& event);
		void OnTextCtrl3Text(wxCommandEvent& event);
		void OnTextCtrl4Text(wxCommandEvent& event);
		void OnTextCtrl5Text(wxCommandEvent& event);
		void OnTextCtrl6Text(wxCommandEvent& event);
		void OnTextCtrl7Text(wxCommandEvent& event);
		void OnTextCtrl8Text(wxCommandEvent& event);
		void OnTextCtrl9Text(wxCommandEvent& event);
		void OnTextCtrl10Text(wxCommandEvent& event);
		void OnTextCtrl1Text(wxCommandEvent& event);
		void OnPanel1LeftDClick(wxMouseEvent& event);
		//*)

		DECLARE_EVENT_TABLE()
};

#endif
