/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     FLEXCABLE.h
//
// DESCRIPTION
//     GUI for the construction of cable_spec file for cable type: Flexcable
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////
#ifndef FLEXCABLE_H
#define FLEXCABLE_H

//(*Headers(FLEXCABLE)
#include <wx/notebook.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/panel.h>
#include <wx/button.h>
#include <wx/dialog.h>
//*)

// Additional wxWidgets Includes
#include <wx/msgdlg.h>
#include <wx/dcclient.h>

// Application Includes
#include "UTILITIES.h"
#include "MESH_LAPLACE.h"
#include "MESH_PARAM.h"
#include "RUN_STATUS.h"
#include "FD_ESR.h"

// Standard Library Header Files
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

class flexcableSchematicPane : public wxPanel
{

public:
    flexcableSchematicPane(wxFrame* parent);

    void paintEvent(wxPaintEvent & evt);
    void paintNow();

    void render(wxDC& dc);

    // some useful events
    /*
     void mouseMoved(wxMouseEvent& event);
     void mouseDown(wxMouseEvent& event);
     void mouseWheelMoved(wxMouseEvent& event);
     void mouseReleased(wxMouseEvent& event);
     void rightClick(wxMouseEvent& event);
     void mouseLeftWindow(wxMouseEvent& event);
     void keyPressed(wxKeyEvent& event);
     void keyReleased(wxKeyEvent& event);
     */

    DECLARE_EVENT_TABLE()
};

class FLEXCABLE: public wxDialog
{
	public:

		FLEXCABLE(wxWindow* parent,wxWindowID id=wxID_ANY,const wxPoint& pos=wxDefaultPosition,const wxSize& size=wxDefaultSize);
		virtual ~FLEXCABLE();

		wxString pathToMOD;
		wxString MODname;
		void SetPathToMOD(wxString, wxString);

        void WriteCableFile(wxString);

        void paintNow();
        void render(wxDC&  dc);
        void flexcableSchematic();
        void OnHighlightSchematic(wxCommandEvent & event);

        flexcableSchematicPane *drawPane;

        wxString FD_ESR6;
        wxString FD_ESR7;
        wxString FD_ESR8;
        wxString FD_ESR9;
        wxString FD_ESR10;

        wxString MESH_PARAM1;
        wxString MESH_PARAM2;

		//(*Declarations(FLEXCABLE)
		wxTextCtrl* TextCtrl4;
		wxNotebook* Notebook1;
		wxStaticText* StaticText13;
		wxStaticText* StaticText2;
		wxStaticText* StaticText14;
		wxButton* Button1;
		wxStaticText* StaticText6;
		wxTextCtrl* TextCtrl6;
		wxPanel* Panel1;
		wxStaticText* StaticText1;
		wxStaticText* StaticText3;
		wxButton* Button2;
		wxPanel* Panel3;
		wxCheckBox* CheckBox1;
		wxButton* Button3;
		wxTextCtrl* TextCtrl14;
		wxStaticText* StaticText5;
		wxTextCtrl* TextCtrl13;
		wxTextCtrl* TextCtrl2;
		wxTextCtrl* TextCtrl1;
		wxStaticText* StaticText15;
		wxStaticText* StaticText12;
		wxPanel* Panel2;
		wxTextCtrl* TextCtrl5;
		wxStaticText* StaticText17;
		wxStaticText* StaticText4;
		wxTextCtrl* TextCtrl3;
		wxTextCtrl* TextCtrl12;
		wxStaticText* StaticText16;
		//*)

	protected:

		//(*Identifiers(FLEXCABLE)
		static const long ID_STATICTEXT1;
		static const long ID_TEXTCTRL1;
		static const long ID_STATICTEXT16;
		static const long ID_TEXTCTRL13;
		static const long ID_STATICTEXT2;
		static const long ID_TEXTCTRL2;
		static const long ID_STATICTEXT3;
		static const long ID_TEXTCTRL3;
		static const long ID_STATICTEXT4;
		static const long ID_TEXTCTRL4;
		static const long ID_STATICTEXT5;
		static const long ID_TEXTCTRL5;
		static const long ID_STATICTEXT6;
		static const long ID_TEXTCTRL6;
		static const long ID_STATICTEXT17;
		static const long ID_TEXTCTRL14;
		static const long ID_PANEL2;
		static const long ID_STATICTEXT12;
		static const long ID_TEXTCTRL12;
		static const long ID_STATICTEXT13;
		static const long ID_STATICTEXT14;
		static const long ID_STATICTEXT15;
		static const long ID_CHECKBOX1;
		static const long ID_PANEL3;
		static const long ID_NOTEBOOK1;
		static const long ID_BUTTON1;
		static const long ID_BUTTON2;
		static const long ID_BUTTON3;
		static const long ID_PANEL1;
		//*)

	private:

		//(*Handlers(FLEXCABLE)
		void OnButton3Click(wxCommandEvent& event);
		void OnButton1Click(wxCommandEvent& event);
		void OnButton2Click(wxCommandEvent& event);
		void OnCheckBox1Click(wxCommandEvent& event);
		void OnTextCtrl2Text(wxCommandEvent& event);
		void OnTextCtrl3Text(wxCommandEvent& event);
		void OnTextCtrl4Text(wxCommandEvent& event);
		void OnTextCtrl5Text(wxCommandEvent& event);
		void OnTextCtrl6Text(wxCommandEvent& event);
		void OnTextCtrl12Text(wxCommandEvent& event);
		void OnTextCtrl7Text(wxCommandEvent& event);
		void OnTextCtrl8Text(wxCommandEvent& event);
		void OnTextCtrl9Text(wxCommandEvent& event);
		void OnTextCtrl10Text(wxCommandEvent& event);
		void OnTextCtrl11Text(wxCommandEvent& event);
		void OnPanel1Paint(wxPaintEvent& event);
		void OnTextCtrl13Text(wxCommandEvent& event);
		void OnNotebook1PageChanging(wxNotebookEvent& event);
		void OnPanel1LeftDClick(wxMouseEvent& event);
		void OnComboBox1Selected(wxCommandEvent& event);
		void OnCheckBox3Click(wxCommandEvent& event);
		//*)

		DECLARE_EVENT_TABLE()
};

#endif
