/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     SPICEMODEL.cpp
//
// DESCRIPTION
//     Application for the creation of SPice models
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////
#ifndef SPICEMODEL_H
#define SPICEMODEL_H

//(*Headers(SPICEMODEL)
#include <wx/notebook.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/radiobut.h>
#include <wx/panel.h>
#include <wx/button.h>
#include <wx/dialog.h>
//*)

// Additional wxWidgets Includes
#include <wx/msgdlg.h>
#include <wx/dirctrl.h>
#include <wx/treectrl.h>
#include <iostream>
#include <string>

// Application Includes
#include "FileSelectorCtrl.h"
#include "BUNDLE_SCHEMATIC.h"
#include "RUN_STATUS.h"
#include "UTILITIES.h"

// Standard Library Header Files

class SPICEMODEL: public wxDialog
{
	public:

		SPICEMODEL(wxWindow* parent,wxWindowID id=wxID_ANY,const wxPoint& pos=wxDefaultPosition,const wxSize& size=wxDefaultSize);
		virtual ~SPICEMODEL();

        void paintEvent(wxPaintEvent & evt);
        void paintNow();

        void render(wxDC& dc);

		void OnTreeSelectionChanged(wxTreeEvent& evt);
		void SetPathForDir(wxString, wxString );

        void SetPathToMOD(wxString, wxString);
        wxString PathToMOD;
        wxString MODname;

        wxSpecificDirCtrl *tree;

        void renderBundle(wxString);
        void FormatPanel(wxString);




//        void DrawBundleCable(wxString, wxString, wxString);
//        void ConstructBundleCsection(wxString, wxString, wxString);
//        void DrawBundleCsection();


        void viewMesh();

		//(*Declarations(SPICEMODEL)
		wxStaticText* StaticText10;
		wxStaticText* StaticText9;
		wxTextCtrl* TextCtrl4;
		wxRadioButton* RadioButton1;
		wxPanel* Panel5;
		wxNotebook* Notebook1;
		wxStaticText* StaticText13;
		wxStaticText* StaticText2;
		wxPanel* Panel4;
		wxStaticText* StaticText14;
		wxRadioButton* RadioButton2;
		wxButton* Button1;
		wxStaticText* StaticText6;
		wxTextCtrl* TextCtrl6;
		wxStaticText* StaticText8;
		wxStaticText* StaticText11;
		wxPanel* Panel1;
		wxStaticText* StaticText1;
		wxStaticText* StaticText3;
		wxButton* Button2;
		wxCheckBox* CheckBox1;
		wxPanel* Panel3;
		wxStaticText* StaticText21;
		wxStaticText* ConductorNumber;
		wxButton* Button3;
		wxStaticText* StaticText5;
		wxTextCtrl* TextCtrl15;
		wxStaticText* StaticText7;
		wxTextCtrl* TextCtrl13;
		wxTextCtrl* TextCtrl8;
		wxTextCtrl* TextCtrl2;
		wxTextCtrl* TextCtrl7;
		wxTextCtrl* TextCtrl1;
		wxStaticText* StaticText15;
		wxStaticText* StaticText12;
		wxTextCtrl* TextCtrl9;
		wxPanel* Panel2;
		wxTextCtrl* TextCtrl5;
		wxStaticText* StaticText4;
		wxTextCtrl* TextCtrl3;
		wxTextCtrl* TextCtrl10;
		wxTextCtrl* TextCtrl12;
		wxTextCtrl* TextCtrl11;
		wxStaticText* StaticText16;
		//*)

	protected:

		//(*Identifiers(SPICEMODEL)
		static const long ID_STATICTEXT1;
		static const long ID_TEXTCTRL1;
		static const long ID_PANEL2;
		static const long ID_STATICTEXT21;
		static const long ID_TEXTCTRL15;
		static const long ID_STATICTEXT2;
		static const long ID_TEXTCTRL2;
		static const long ID_STATICTEXT3;
		static const long ID_STATICTEXT4;
		static const long ID_STATICTEXT5;
		static const long ID_TEXTCTRL3;
		static const long ID_STATICTEXT6;
		static const long ID_TEXTCTRL4;
		static const long ID_STATICTEXT7;
		static const long ID_TEXTCTRL5;
		static const long ID_STATICTEXT8;
		static const long ID_TEXTCTRL6;
		static const long ID_PANEL3;
		static const long ID_STATICTEXT9;
		static const long ID_TEXTCTRL7;
		static const long ID_STATICTEXT10;
		static const long ID_TEXTCTRL8;
		static const long ID_STATICTEXT11;
		static const long ID_TEXTCTRL9;
		static const long ID_PANEL4;
		static const long ID_STATICTEXT17;
		static const long ID_CHECKBOX1;
		static const long ID_STATICTEXT12;
		static const long ID_TEXTCTRL10;
		static const long ID_STATICTEXT13;
		static const long ID_RADIOBUTTON1;
		static const long ID_RADIOBUTTON2;
		static const long ID_STATICTEXT14;
		static const long ID_TEXTCTRL11;
		static const long ID_STATICTEXT15;
		static const long ID_TEXTCTRL12;
		static const long ID_STATICTEXT16;
		static const long ID_TEXTCTRL13;
		static const long ID_PANEL5;
		static const long ID_NOTEBOOK1;
		static const long ID_BUTTON1;
		static const long ID_BUTTON2;
		static const long ID_BUTTON3;
		static const long ID_PANEL1;
		//*)

	private:

		//(*Handlers(SPICEMODEL)
		void OnTextCtrl13Text(wxCommandEvent& event);
		void OnPanel1Paint(wxPaintEvent& event);
		void OnButton3Click(wxCommandEvent& event);
		void OnButton2Click(wxCommandEvent& event);
		void OnButton1Click(wxCommandEvent& event);
		void OnTextCtrl15Text(wxCommandEvent& event);
		void OnTextCtrl2Text(wxCommandEvent& event);
		void OnTextCtrl3Text(wxCommandEvent& event);
		void OnTextCtrl5Text(wxCommandEvent& event);
		void OnTextCtrl4Text(wxCommandEvent& event);
		void OnTextCtrl6Text(wxCommandEvent& event);
		void OnTextCtrl7Text(wxCommandEvent& event);
		void OnTextCtrl8Text(wxCommandEvent& event);
		void OnTextCtrl9Text(wxCommandEvent& event);
		void OnNotebook1PageChanged(wxNotebookEvent& event);
		void OnNotebook1PageChanged1(wxNotebookEvent& event);
		void OnTextCtrl13Text1(wxCommandEvent& event);
		//*)

        void PopulateTheDirectoryTree ( wxString);
        int DrawAPicture = 0;

        wxString bundleFile;
        wxString bundleSelected;

		DECLARE_EVENT_TABLE()
};

#endif
