/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     UTILITIES.h
//
// DESCRIPTION
//     Utility Functions:
//
//     check_is_empty           check if input field is left blank
//     check_is numeric:        check if input parameters are numerical values
//     check_is_numeric_list:   check if input is space delimited numerical list
//     check_filename_valid:    check if filename is valid
//     check_is_greater_than:   check if arguement1 is > than arguement2
//     check_is_less_than:      check if arguement1 is < than arguement2
//     clean_temp_files         clean up run folder
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////


// wxWidgets Includes
#include <wx/textctrl.h>
#include <wx/tokenzr.h>
#include <wx/filename.h>
#include <wx/filefn.h>

// Application Includes

#ifdef __linux__

#endif

void check_is_empty(wxTextCtrl *TxtBox, int *IsError);

void check_is_numeric(wxTextCtrl *TxtBox, int *IsError);

void check_is_numeric_list(wxTextCtrl *TxtBox, int *IsError);

void check_filename_valid(wxTextCtrl *TxtBox, int *IsError);

void check_is_positive (wxTextCtrl *TxtBox, int *IsError);

void check_is_less_than (wxTextCtrl *TxtBox1, wxTextCtrl *TxtBox2, int *IsError);

void check_is_greater_than (wxTextCtrl *TxtBox1, wxTextCtrl *TxtBox2, int *IsError);

void clean_temp_files();

