/////////////////////////////////////////////////////////////////////////////////
//
// This file is part of SACAMOS, cable models for EMI simulations in SPICE.
// It was developed by the University of Nottingham and the Netherlands Aerospace
// Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
//
// Copyright (C) 2015 - 2017 University of Nottingham
//
// SACAMOS is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// SACAMOS is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// A copy of the GNU General Public License version 3 can be found in the
// file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
//
// wxWidgets is currently licenced under the "wxWindows Library Licence".
// A copy of the wxWindows Library Licence, Version 3.1 can be found in the file
// wxWindows_Library_Licence_v3-1 in the root or at:
// <https://www.wxwidgets.org/about/licence/>
//
// The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
//
// File Contents:
//
// NAME
//     UTP.cpp
//
// DESCRIPTION
//     GUI for the construction of cable_spec file for cable type: Unshielded Twisted Pair
//
// AUTHOR(S)
//     Steve Greedy
//
/////////////////////////////////////////////////////////////////////////////////
#include "UTP.h"

//(*InternalHeaders(UTP)
#include <wx/settings.h>
#include <wx/intl.h>
#include <wx/string.h>
//*)

//(*IdInit(UTP)
const long UTP::ID_STATICTEXT1 = wxNewId();
const long UTP::ID_TEXTCTRL1 = wxNewId();
const long UTP::ID_STATICTEXT2 = wxNewId();
const long UTP::ID_TEXTCTRL2 = wxNewId();
const long UTP::ID_STATICTEXT4 = wxNewId();
const long UTP::ID_TEXTCTRL4 = wxNewId();
const long UTP::ID_STATICTEXT3 = wxNewId();
const long UTP::ID_TEXTCTRL3 = wxNewId();
const long UTP::ID_STATICTEXT5 = wxNewId();
const long UTP::ID_TEXTCTRL5 = wxNewId();
const long UTP::ID_PANEL2 = wxNewId();
const long UTP::ID_STATICTEXT11 = wxNewId();
const long UTP::ID_TEXTCTRL11 = wxNewId();
const long UTP::ID_STATICTEXT12 = wxNewId();
const long UTP::ID_STATICTEXT13 = wxNewId();
const long UTP::ID_STATICTEXT14 = wxNewId();
const long UTP::ID_CHECKBOX1 = wxNewId();
const long UTP::ID_PANEL3 = wxNewId();
const long UTP::ID_NOTEBOOK1 = wxNewId();
const long UTP::ID_STATICTEXT15 = wxNewId();
const long UTP::ID_CHECKBOX2 = wxNewId();
const long UTP::ID_STATICTEXT17 = wxNewId();
const long UTP::ID_COMBOBOX1 = wxNewId();
const long UTP::ID_STATICTEXT16 = wxNewId();
const long UTP::ID_CHECKBOX3 = wxNewId();
const long UTP::ID_BUTTON1 = wxNewId();
const long UTP::ID_BUTTON2 = wxNewId();
const long UTP::ID_BUTTON3 = wxNewId();
const long UTP::ID_PANEL1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(UTP,wxDialog)
	//(*EventTable(UTP)
	//*)
END_EVENT_TABLE()

UTP::UTP(wxWindow* parent,wxWindowID id,const wxPoint& pos,const wxSize& size)
{
	//(*Initialize(UTP)
	wxBoxSizer* BoxSizer4;
	wxStaticBoxSizer* StaticBoxSizer2;
	wxFlexGridSizer* FlexGridSizer3;
	wxFlexGridSizer* FlexGridSizer5;
	wxFlexGridSizer* FlexGridSizer2;
	wxBoxSizer* BoxSizer2;
	wxBoxSizer* BoxSizer1;
	wxFlexGridSizer* FlexGridSizer6;
	wxStaticBoxSizer* StaticBoxSizer1;
	wxFlexGridSizer* FlexGridSizer1;
	wxBoxSizer* BoxSizer3;

	Create(parent, wxID_ANY, _("SACAMOS: TWISTED PAIR MODEL"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("wxID_ANY"));
	FlexGridSizer1 = new wxFlexGridSizer(0, 2, 0, 0);
	StaticBoxSizer1 = new wxStaticBoxSizer(wxVERTICAL, this, _("Twisted Pair Parameters"));
	FlexGridSizer2 = new wxFlexGridSizer(0, 1, 0, 0);
	BoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
	StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("Cable Name"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
	BoxSizer1->Add(StaticText1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl1 = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	BoxSizer1->Add(TextCtrl1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(BoxSizer1, 1, wxALL|wxEXPAND, 5);
	BoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
	Notebook1 = new wxNotebook(this, ID_NOTEBOOK1, wxDefaultPosition, wxDefaultSize, 0, _T("ID_NOTEBOOK1"));
	Panel2 = new wxPanel(Notebook1, ID_PANEL2, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANEL2"));
	FlexGridSizer3 = new wxFlexGridSizer(0, 2, 0, 0);
	StaticText2 = new wxStaticText(Panel2, ID_STATICTEXT2, _("Conductor Radius (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT2"));
	FlexGridSizer3->Add(StaticText2, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl2 = new wxTextCtrl(Panel2, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL2"));
	FlexGridSizer3->Add(TextCtrl2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText4 = new wxStaticText(Panel2, ID_STATICTEXT4, _("Dielectric Radius (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT4"));
	FlexGridSizer3->Add(StaticText4, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl4 = new wxTextCtrl(Panel2, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL4"));
	FlexGridSizer3->Add(TextCtrl4, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText3 = new wxStaticText(Panel2, ID_STATICTEXT3, _("Conductor Separation (m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT3"));
	FlexGridSizer3->Add(StaticText3, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl3 = new wxTextCtrl(Panel2, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL3"));
	FlexGridSizer3->Add(TextCtrl3, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText5 = new wxStaticText(Panel2, ID_STATICTEXT5, _("Conductor Conductivity (S/m)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT5"));
	FlexGridSizer3->Add(StaticText5, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl5 = new wxTextCtrl(Panel2, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL5"));
	FlexGridSizer3->Add(TextCtrl5, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	Panel2->SetSizer(FlexGridSizer3);
	FlexGridSizer3->Fit(Panel2);
	FlexGridSizer3->SetSizeHints(Panel2);
	Panel3 = new wxPanel(Notebook1, ID_PANEL3, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANEL3"));
	BoxSizer4 = new wxBoxSizer(wxVERTICAL);
	FlexGridSizer5 = new wxFlexGridSizer(0, 2, 0, 0);
	StaticText11 = new wxStaticText(Panel3, ID_STATICTEXT11, _("Dielectric Permittivity"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT11"));
	FlexGridSizer5->Add(StaticText11, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	TextCtrl11 = new wxTextCtrl(Panel3, ID_TEXTCTRL11, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL11"));
	FlexGridSizer5->Add(TextCtrl11, 1, wxALL|wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL, 5);
	StaticText12 = new wxStaticText(Panel3, ID_STATICTEXT12, _("Or"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT12"));
	FlexGridSizer5->Add(StaticText12, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText13 = new wxStaticText(Panel3, ID_STATICTEXT13, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT13"));
	FlexGridSizer5->Add(StaticText13, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	StaticText14 = new wxStaticText(Panel3, ID_STATICTEXT14, _("Frequency Dependant Model"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT14"));
	FlexGridSizer5->Add(StaticText14, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	CheckBox1 = new wxCheckBox(Panel3, ID_CHECKBOX1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_CHECKBOX1"));
	CheckBox1->SetValue(false);
	FlexGridSizer5->Add(CheckBox1, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	BoxSizer4->Add(FlexGridSizer5, 1, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
	Panel3->SetSizer(BoxSizer4);
	BoxSizer4->Fit(Panel3);
	BoxSizer4->SetSizeHints(Panel3);
	Notebook1->AddPage(Panel2, _("General"), false);
	Notebook1->AddPage(Panel3, _("Material"), false);
	BoxSizer3->Add(Notebook1, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer2->Add(BoxSizer3, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer6 = new wxFlexGridSizer(0, 2, 0, 0);
	StaticText15 = new wxStaticText(this, ID_STATICTEXT15, _("Use Laplace"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT15"));
	FlexGridSizer6->Add(StaticText15, 0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	CheckBox2 = new wxCheckBox(this, ID_CHECKBOX2, _("Yes"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_CHECKBOX2"));
	CheckBox2->SetValue(false);
	FlexGridSizer6->Add(CheckBox2, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	StaticText17 = new wxStaticText(this, ID_STATICTEXT17, _("View Mesh"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT17"));
	FlexGridSizer6->Add(StaticText17, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	ComboBox1 = new wxComboBox(this, ID_COMBOBOX1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, 0, 0, wxDefaultValidator, _T("ID_COMBOBOX1"));
	FlexGridSizer6->Add(ComboBox1, 1, wxALL, 5);
	StaticText16 = new wxStaticText(this, ID_STATICTEXT16, _("Refine Laplace Mesh"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT16"));
	FlexGridSizer6->Add(StaticText16, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	CheckBox3 = new wxCheckBox(this, ID_CHECKBOX3, _("Yes"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_CHECKBOX3"));
	CheckBox3->SetValue(false);
	CheckBox3->Disable();
	FlexGridSizer6->Add(CheckBox3, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	FlexGridSizer2->Add(FlexGridSizer6, 2, wxALL|wxEXPAND, 5);
	StaticBoxSizer1->Add(FlexGridSizer2, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	BoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
	Button1 = new wxButton(this, ID_BUTTON1, _("SAVE"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	BoxSizer2->Add(Button1, 1, wxALL|wxALIGN_BOTTOM, 5);
	Button2 = new wxButton(this, ID_BUTTON2, _("BUILD"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
	BoxSizer2->Add(Button2, 1, wxALL|wxALIGN_BOTTOM, 5);
	Button3 = new wxButton(this, ID_BUTTON3, _("CLOSE"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
	BoxSizer2->Add(Button3, 1, wxALL|wxALIGN_BOTTOM, 5);
	StaticBoxSizer1->Add(BoxSizer2, 1, wxALL|wxEXPAND, 5);
	FlexGridSizer1->Add(StaticBoxSizer1, 1, wxALL|wxEXPAND, 5);
	StaticBoxSizer2 = new wxStaticBoxSizer(wxHORIZONTAL, this, _("Cross Section"));
	Panel1 = new wxPanel(this, ID_PANEL1, wxDefaultPosition, wxSize(500,500), wxTAB_TRAVERSAL, _T("ID_PANEL1"));
	Panel1->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
	Panel1->SetToolTip(_("Double Click to Reset Schematic"));
	StaticBoxSizer2->Add(Panel1, 1, wxALL|wxALIGN_TOP, 5);
	FlexGridSizer1->Add(StaticBoxSizer2, 1, wxALL|wxALIGN_TOP|wxALIGN_CENTER_HORIZONTAL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);

	Connect(ID_TEXTCTRL2,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&UTP::OnTextCtrl2Text);
	Connect(ID_TEXTCTRL4,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&UTP::OnTextCtrl4Text);
	Connect(ID_TEXTCTRL3,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&UTP::OnTextCtrl3Text);
	Connect(ID_TEXTCTRL5,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&UTP::OnTextCtrl5Text);
	Connect(ID_TEXTCTRL11,wxEVT_COMMAND_TEXT_UPDATED,(wxObjectEventFunction)&UTP::OnTextCtrl11Text);
	Connect(ID_CHECKBOX1,wxEVT_COMMAND_CHECKBOX_CLICKED,(wxObjectEventFunction)&UTP::OnCheckBox1Click);
	Connect(ID_NOTEBOOK1,wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING,(wxObjectEventFunction)&UTP::OnNotebook1PageChanging);
	Connect(ID_COMBOBOX1,wxEVT_COMMAND_COMBOBOX_SELECTED,(wxObjectEventFunction)&UTP::OnComboBox1Selected);
	Connect(ID_CHECKBOX3,wxEVT_COMMAND_CHECKBOX_CLICKED,(wxObjectEventFunction)&UTP::OnCheckBox3Click);
	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&UTP::OnButton1Click);
	Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&UTP::OnButton2Click);
	Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&UTP::OnButton3Click);
	Panel1->Connect(wxEVT_PAINT,(wxObjectEventFunction)&UTP::OnPanel1Paint,0,this);
	Panel1->Connect(wxEVT_LEFT_DCLICK,(wxObjectEventFunction)&UTP::OnPanel1LeftDClick,0,this);
	Panel1->Connect(wxEVT_MOUSEWHEEL,(wxObjectEventFunction)&UTP::OnPanel1MouseWheel,0,this);
	//*)

    TextCtrl2->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
    TextCtrl3->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
    TextCtrl4->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
    TextCtrl5->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
//    TextCtrl6->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
//    TextCtrl7->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
//    TextCtrl8->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
//    TextCtrl9->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
//    TextCtrl10->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
    TextCtrl11->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
    CheckBox1->Bind(wxEVT_SET_FOCUS,(wxObjectEventFunction)&UTP::OnHighlightSchematic, this);
}

UTP::~UTP()
{
	//(*Destroy(UTP)
	//*)
}

void UTP::SetPathToMOD(wxString Path, wxString Name)
{
    pathToMOD << Path;
    MODname << Name;
}

void UTP::OnButton1Click(wxCommandEvent& event)
{
    int IsError = 0;

    wxString fileName = TextCtrl1->GetValue();

    TextCtrl1->SetBackgroundColour(wxColor(255,255,255));

    if (fileName == "")
    {
        wxMessageBox("Please Specify a Cable Name" , _("Missing Cable Name"));
        return;
    }

    check_is_empty(TextCtrl2, &IsError);
    check_is_empty(TextCtrl3, &IsError);
    check_is_empty(TextCtrl4, &IsError);
    check_is_empty(TextCtrl5, &IsError);

    if (CheckBox1->GetValue())
    {
//        check_is_empty(TextCtrl6, &IsError);
//        check_is_empty(TextCtrl7, &IsError);
//        check_is_empty(TextCtrl8, &IsError);
//        check_is_empty(TextCtrl9, &IsError);
//        check_is_empty(TextCtrl10, &IsError);
    }
    else
    {
        check_is_empty(TextCtrl11, &IsError);
    }

    if (IsError ==1)
    {
        return;
    }

    if ( IsError == 1)
    {
        wxMessageBox("Please fix highlighted errors" , _("Non numeric entry"));
        return;
    }

    WriteCableFile (fileName);

    Button1->Disable();
}

void UTP::OnCheckBox1Click(wxCommandEvent& event)
{
    TextCtrl11->Disable();
//    TextCtrl6->Enable();
//    TextCtrl7->Enable();
//    TextCtrl8->Enable();
//    TextCtrl9->Enable();
//    TextCtrl10->Enable();

    FD_ESR *winFD_ESR = new FD_ESR(this);

    if ( winFD_ESR->ShowModal() == wxID_OK )
    {
//        FD_ESR1 = winFD_ESR->TextCtrl1->GetValue();
//        FD_ESR2 = winFD_ESR->TextCtrl2->GetValue();
//        FD_ESR3 = winFD_ESR->TextCtrl3->GetValue();
//        FD_ESR4 = winFD_ESR->TextCtrl4->GetValue();
//        FD_ESR5 = winFD_ESR->TextCtrl5->GetValue();

        FD_ESR6 = winFD_ESR->TextCtrl6->GetValue();
        FD_ESR7 = winFD_ESR->TextCtrl7->GetValue();
        FD_ESR8 = winFD_ESR->TextCtrl8->GetValue();
        FD_ESR9 = winFD_ESR->TextCtrl9->GetValue();
        FD_ESR10= winFD_ESR->TextCtrl10->GetValue();

        FD_ORDER = winFD_ESR->TextCtrl11->GetValue();
        FD_lin   = winFD_ESR->RadioButton1->GetValue();
        FD_Fmin  = winFD_ESR->TextCtrl12->GetValue();
        FD_Fmax  = winFD_ESR->TextCtrl12->GetValue();
        FD_Num   = winFD_ESR->TextCtrl12->GetValue();

    }
    winFD_ESR->Destroy();

}

void UTP::OnButton2Click(wxCommandEvent& event)
{
    //clean up directory
    clean_temp_files();

    //CABLE BUIDER
    wxString fileName = TextCtrl1->GetValue();
    wxString fineName = fileName;

    std::stringstream stream;
    stream  << "\"..\\BIN\\cable_model_builder.exe\""
            << " "
            << "..\\" + MODname + "\\CABLE\\"
            << fileName ;

    system(stream.str().c_str());

    ifstream runStatus;

    // Display Run Status
    RUN_STATUS *run_status = new RUN_STATUS(this);
    run_status->ShowModal();

    Button1->Enable();

    if (CheckBox2->GetValue())
    {
        // Identify all mesh files for viewing
        //struct _finddata_t fileinfo;
        //long hFile;
        //wxString meshFileName = "*.msh.vtk";
        wxString pattern = "*.msh.vtk";
        //wxString pattern((meshFileName));
        wxString file;

        ComboBox1->Clear();

        file = wxFindFirstFile(pattern);
        while ( !file.empty())
        {
                wxString meshfile = file;
                size_t extindex = meshfile.find_first_of (".");

                meshfile  = meshfile.substr(0, extindex);

                ComboBox1->Append(meshfile);
        }

//        if (( hFile = _findfirst( pattern, &fileinfo )) != -1 )
//        {
//            do
//            {
//                wxString meshfile = fileinfo.name;
//                size_t extindex = meshfile.find_first_of (".");
//
//                meshfile  = meshfile.substr(0, extindex);
//
//                ComboBox1->Append(meshfile);
//
//            } while ( _findnext( hFile, &fileinfo) == 0);
//        }
//        _findclose( hFile);

        CheckBox3->Enable();
    }
}

void UTP::OnButton3Click(wxCommandEvent& event)
{
    //clean up directory
    clean_temp_files();

    Close();
}

void UTP::OnTextCtrl2Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl2, &IsError);
    check_is_positive (TextCtrl2, &IsError);
}

void UTP::OnTextCtrl3Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl3, &IsError);
    check_is_positive (TextCtrl3, &IsError);

    double value_tb3, value_tb4;
    wxString contents_tb3, contents_tb4;

    contents_tb3 = TextCtrl3->GetValue();
    contents_tb4 = TextCtrl4->GetValue();

    contents_tb3.ToDouble(&value_tb3);
    contents_tb4.ToDouble(&value_tb4);

    if (value_tb3 <= (value_tb4 + value_tb4))
    {
        TextCtrl3->SetBackgroundColour(wxColor(255,153,153));
        TextCtrl3->SetFocus();
        IsError = 1;
    }
}

void UTP::OnTextCtrl4Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl4, &IsError);
    check_is_positive (TextCtrl4, &IsError);
    check_is_less_than(TextCtrl4, TextCtrl2, &IsError);
}

void UTP::OnTextCtrl5Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl5, &IsError);
    check_is_positive (TextCtrl5, &IsError);
}

void UTP::OnTextCtrl11Text(wxCommandEvent& event)
{
    int IsError = 0;
    check_is_numeric_list(TextCtrl11, &IsError);
}

//void UTP::OnTextCtrl6Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric_list(TextCtrl6, &IsError);
//}
//
//void UTP::OnTextCtrl7Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric_list(TextCtrl7, &IsError);
//}
//
//void UTP::OnTextCtrl8Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric_list(TextCtrl8, &IsError);
//}
//
//void UTP::OnTextCtrl9Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric_list(TextCtrl9, &IsError);
//}
//
//void UTP::OnTextCtrl10Text(wxCommandEvent& event)
//{
//    int IsError = 0;
//    check_is_numeric_list(TextCtrl10, &IsError);
//}

void UTP::WriteCableFile (wxString fileName)
{

    ofstream cableFile;

    wxString fullFileName = pathToMOD + "//CABLE//" + fileName + ".cable_spec";

    cableFile.open (fullFileName);

    if (cableFile.is_open())
    {
        // minimum requirements for type UTP
        cableFile <<"# MOD_cable_lib_dir \n";
        cableFile <<"../"<< MODname <<"/CABLE/"<<"\n";
        cableFile <<"Twisted_pair\n";
        cableFile <<"2 \t#number of conductors \n";
        cableFile <<"4 \t#number of parameters \n";

        cableFile << TextCtrl2->GetValue() <<" \t# parameter 1: conductor radius  (m)\n";
        cableFile << TextCtrl3->GetValue() <<" \t# parameter 2: conductor separation  (m)\n";
        cableFile << TextCtrl4->GetValue() <<" \t# parameter 3: dielectric radius (m)\n";
        cableFile << TextCtrl5->GetValue() <<" \t# parameter 4: inner conductor electric conductivity (m)\n";
        cableFile << "1 \t# number of frequency dependent parameters\n";

        if (CheckBox1->GetValue()){
            cableFile << "# Dielectric relative permittivity model follows\n";
            cableFile << FD_ESR6 <<"\t # w normalisation constant\n";
            cableFile << FD_ESR7 <<"\t # a order, a coefficients follow below\n";
            cableFile << FD_ESR8 <<"\n";
            cableFile << FD_ESR9 <<"\t # b order, b coefficients follow below\n";
            cableFile << FD_ESR10 <<"\n";

//            cableFile << "# Dielectric relative permittivity model follows\n";
//            cableFile << TextCtrl6->GetValue() <<" \t# w normalisation constant\n";
//            cableFile << TextCtrl7->GetValue() <<" \t# a order, a coefficients follow below\n";
//            cableFile << TextCtrl8->GetValue() <<" \n";
//            cableFile << TextCtrl9->GetValue() <<" \t# b order, b coefficients follow below\n";
//            cableFile << TextCtrl10->GetValue() <<" \n";
        }

        else{
            cableFile << "\t# Dielectric relative permittivity model follows\n";
            cableFile << "1.0 \t# w normalisation constant\n";
            cableFile << "0 \t# a order, a coefficients follow below\n";
            cableFile << TextCtrl11->GetValue()<<"\n";
            cableFile << "0 \t# b order, b coefficients follow below\n";
            cableFile << "1.0\n";
        }
    }

    if (CheckBox1->GetValue()){

        cableFile << FD_ORDER <<"\t # order for filter fitting\n";
        if (FD_lin)
        {
            cableFile << "lin   # frequency range type for filter fitting type (lin or dB)";
        }
        else
        {
            cableFile << "log   # frequency range type for filter fitting type (lin or dB)";
        }
        cableFile << FD_Fmin <<" "<< FD_Fmax <<" "<< FD_ORDER <<"\t # fmin fmax number_of_frequencies for filter fitting";
    }

    if (CheckBox2->GetValue())
    {
        cableFile << "use_laplace\n";
    }
    else
    {
        cableFile << "no_laplace\n";
    }


    if (CheckBox3->GetValue())
    {
        cableFile << "Laplace_boundary_constant\n";
        cableFile << MESH_PARAM1 <<"\n";
        cableFile << "Laplace_surface_mesh_constant\n";
        cableFile << MESH_PARAM2 <<"\n";

        CheckBox3 ->SetValue(false);
    }

    cableFile.close();
}

void UTP::OnPanel1Paint(wxPaintEvent& event)
{
    utpSchematic();
}

void UTP::utpSchematic()
{
    int size_h, size_v;
    Panel1->GetSize(&size_h, &size_v );

    wxClientDC dc(Panel1);
    dc.Clear();

    // Draw centre lines
    dc.SetPen( wxPen( wxColor(224,224,224),1 , wxDOT_DASH) );
    dc.DrawLine( 0, size_v/2, size_h,size_v/2 );
    dc.DrawLine( size_h/2, 0, size_h/2, size_v);

    dc.SetTextForeground(*wxBLUE);
    dc.DrawText("General Twinax Cable", 10, 15);

    float conductor_radius, dielectric_radius;

    conductor_radius = 0.06 * size_h;
    dielectric_radius = 0.075 * size_h;

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxFDIAGONAL_HATCH));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2+(0.1*size_v)), dielectric_radius );

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxFDIAGONAL_HATCH));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2-(0.1*size_v)), dielectric_radius );

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2+(0.1*size_v)), conductor_radius );

    dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
    dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
    dc.DrawCircle( wxPoint(size_h/2,size_v/2-(0.1*size_v)), conductor_radius );
}

void UTP::OnHighlightSchematic(wxCommandEvent& event)
{
    utpSchematic();

    int ID = event.GetId();
    event.Skip(true);

    int size_h, size_v;
    Panel1->GetSize(&size_h, &size_v );

    float conductor_radius, dielectric_radius;

    dielectric_radius = 0.075 * size_h;
    conductor_radius = 0.06 * size_h;

    float x, y;

    wxClientDC dc(Panel1);

    wxPoint arrowHead[3];

    if ( ID == ID_TEXTCTRL2)
    {
        x = conductor_radius * 0.7071;
        y = conductor_radius * 0.7071;
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2-(0.1*size_v)),4 );
        dc.DrawLine( size_h/2,size_v/2-(0.1*size_v), size_h/2+x,size_v/2-(0.1*size_v)-y );

        arrowHead[0] = wxPoint(size_h/2+x,size_v/2-(0.1*size_v)-y);
        arrowHead[1] = wxPoint(size_h/2+x-5,size_v/2-(0.1*size_v)-y);
        arrowHead[2] = wxPoint(size_h/2+x,size_v/2-(0.1*size_v)-y+5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);
    }

    if ( ID == ID_TEXTCTRL3)
    {
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawLine( size_h/2,size_v/2-(0.1*size_v),  size_h/2,size_v/2+(0.1*size_v) );

        arrowHead[0] = wxPoint(size_h/2,size_v/2-(0.1*size_v));
        arrowHead[1] = wxPoint(size_h/2+5,size_v/2-(0.1*size_v)+5);
        arrowHead[2] = wxPoint(size_h/2-5,size_v/2-(0.1*size_v)+5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);

        arrowHead[0] = wxPoint(size_h/2,size_v/2+(0.1*size_v));
        arrowHead[1] = wxPoint(size_h/2+5,size_v/2+(0.1*size_v)-5);
        arrowHead[2] = wxPoint(size_h/2-5,size_v/2+(0.1*size_v)-5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);
    }

    if ( ID == ID_TEXTCTRL4)
    {
        x = dielectric_radius * 0.7071;
        y = dielectric_radius * 0.7071;
        dc.SetPen(wxPen(wxColor(51,51,255), 3));
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2-(0.1*size_v)),4 );
        dc.DrawLine( size_h/2,size_v/2-(0.1*size_v), size_h/2-x,size_v/2-(0.1*size_v)-y );

        arrowHead[0] = wxPoint(size_h/2-x,size_v/2-(0.1*size_v)-y);
        arrowHead[1] = wxPoint(size_h/2-x+5,size_v/2-(0.1*size_v)-y);
        arrowHead[2] = wxPoint(size_h/2-x,size_v/2-(0.1*size_v)-y+5);
        dc.DrawPolygon(WXSIZEOF(arrowHead), arrowHead);
    }

    if ( ID == ID_TEXTCTRL5)
    {
        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2+(0.1*size_v)), conductor_radius );

        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2-(0.1*size_v)), conductor_radius );
    }

    if ( ID == ID_TEXTCTRL11 || ID == ID_CHECKBOX1)
    {
        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxFDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2+(0.1*size_v)), dielectric_radius );

        dc.SetPen( wxPen( wxColor(51,51,255),1 ) );
        dc.SetBrush(wxBrush(wxColor(51,51,255), wxFDIAGONAL_HATCH));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2-(0.1*size_v)), dielectric_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2+(0.1*size_v)), conductor_radius );

        dc.SetPen( wxPen( wxColor(192,192,192),1 ) );
        dc.SetBrush(wxBrush(wxColor(224,224,224), wxSOLID));
        dc.DrawCircle( wxPoint(size_h/2,size_v/2-(0.1*size_v)), conductor_radius );
    }
}



void UTP::OnPanel1LeftDClick(wxMouseEvent& event)
{
    utpSchematic();
}

void UTP::OnNotebook1PageChanging(wxNotebookEvent& event)
{
    utpSchematic();
}

void UTP::OnCheckBox3Click(wxCommandEvent& event)
{
    if (CheckBox3->GetValue())
    {
        MESH_PARAM *winMESH_PARAM = new MESH_PARAM(this);

        if ( winMESH_PARAM->ShowModal() == wxID_OK )
        {
            MESH_PARAM1 = winMESH_PARAM->TextCtrl1->GetValue();
            MESH_PARAM2 = winMESH_PARAM->TextCtrl2->GetValue();
        }
        winMESH_PARAM->Destroy();
    }
    WriteCableFile (TextCtrl1->GetValue());
}

void UTP::OnPanel1MouseWheel(wxMouseEvent& event)
{
    wxString fileName = TextCtrl1->GetValue();

    wxClientDC dc(Panel1);
    dc.Clear();
    zoom_mesh(Panel1,fileName);
}

void UTP::OnComboBox1Selected(wxCommandEvent& event)
{
    wxString meshToView;
    meshToView<<ComboBox1->GetValue();
    draw_mesh(Panel1,meshToView);
}
