!
! This file is part of SACAMOS, State of the Art CAble MOdels for Spice. 
! It was developed by the University of Nottingham and the Netherlands Aerospace 
! Centre (NLR) for ESA under contract number 4000112765/14/NL/HK.
! 
! Copyright (C) 2016-2018 University of Nottingham
! 
! SACAMOS is free software: you can redistribute it and/or modify it under the 
! terms of the GNU General Public License as published by the Free Software 
! Foundation, either version 3 of the License, or (at your option) any later 
! version.
! 
! SACAMOS is distributed in the hope that it will be useful, but 
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
! or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
! for more details.
! 
! A copy of the GNU General Public License version 3 can be found in the 
! file GNU_GPL_v3 in the root or at <http://www.gnu.org/licenses/>.
! 
! SACAMOS uses the EISPACK library (in /SRC/EISPACK). EISPACK is subject to 
! the GNU Lesser General Public License. A copy of the GNU Lesser General Public 
! License version can be found in the file GNU_LGPL in the root of EISPACK 
! (/SRC/EISPACK ) or at <http://www.gnu.org/licenses/>.
! 
! The University of Nottingham can be contacted at: ggiemr@nottingham.ac.uk
!
!
! File Contents:
! MODULE circuit_symbol_module
!
! NAME
!     MODULE circuit_symbol_module
!
! DESCRIPTION
!     Data structures relating to the drawing of circuit symbols for spice transmission line models
!     
! COMMENTS
!     Ngspice and LTspice only at the moment
!
! HISTORY
!    started 15/12/2015 CJS
!
MODULE circuit_symbol_module

USE type_specifications

IMPLICIT NONE

! Gschem (for Ngspice) symbol parameters:
! parameters which define the size of the circuit symbol
! all parameters are defined in mils (1/1000 of an inch)
! see http://wiki.geda-project.org/geda:file_format_spec

! x extent of symbol - this is fixed
integer,parameter    :: xmin_symbol    =200
integer,parameter    :: xmax_symbol    =1800

! y extent of symbol varies with the numbe of conductors
integer,parameter    :: ymin_symbol    =000

integer,parameter    :: ymin_offset    =100
integer,parameter    :: ymax_offset    =100
integer,parameter    :: ymax_text_offset =200

! separation of the conducor pins in the y direction
integer,parameter    :: y_conductor_spacing    =200

! length of connection pins in the x direction
integer,parameter    :: x_pin_length        =200

! We need to check the version number - how do we specify this? Is it related to the version of gEDA?
character(LEN=12),parameter    :: symbol_version='v 20050820 1'


! LTpsice symbol parameters:
! parameters which define the size of the circuit symbol

! It appears that the grid used in the LTspice scematic capture uses a grid with spacing 16 points
! so distances here should be a multiple of 16 points.

! x extent of symbol - this is fixed
integer,parameter    :: LT_xmin_symbol    =16
integer,parameter    :: LT_xmax_symbol    =192

! y extent of symbol varies with the numbe of conductors
integer,parameter    :: LT_ymin_symbol    =00

integer,parameter    :: LT_ymin_offset    =16
integer,parameter    :: LT_ymax_offset    =16
integer,parameter    :: LT_ymax_text_offset =32

! separation of the conducor pins in the y direction
integer,parameter    :: LT_y_conductor_spacing    =32

! length of connection pins in the x direction
integer,parameter    :: LT_x_pin_length        =16

! We need to check the version number - how do we specify this?
character(LEN=9),parameter    :: LT_symbol_version='Version 4'

END MODULE circuit_symbol_module
